@@ -35,6 +35,8 @@
 /*** MODULEINFO
 	<depend>alsa</depend>
 	<support_level>extended</support_level>
+	<deprecated_in>19</deprecated_in>
+	<removed_in>21</removed_in>
  ***/
 
 #include "asterisk.h"
