@@ -77,6 +77,7 @@ extern "C" {
 #include "asterisk/res_srtp.h"
 #include "asterisk/stasis.h"
 #include "asterisk/vector.h"
+#include "asterisk/logger_category.h"
 
 /*! Maximum number of payload types RTP can support. */
 #define AST_RTP_MAX_PT 128
@@ -306,6 +307,8 @@ struct ast_rtp_payload_type {
 #define AST_RTP_RTCP_FMT_FIR	4
 /*! REMB Information (From draft-alvestrand-rmcat-remb-03) */
 #define AST_RTP_RTCP_FMT_REMB	15
+/*! Transport-wide congestion control feedback (From draft-holmer-rmcat-transport-wide-cc-extensions-01) */
+#define AST_RTP_RTCP_FMT_TRANSPORT_WIDE_CC 15
 
 /*!
  * \since 12
@@ -541,6 +544,8 @@ enum ast_rtp_extension {
 	AST_RTP_EXTENSION_UNSUPPORTED = 0,
 	/*! abs-send-time from https://tools.ietf.org/html/draft-alvestrand-rmcat-remb-03 */
 	AST_RTP_EXTENSION_ABS_SEND_TIME,
+	/*! transport-cc from https://tools.ietf.org/html/draft-holmer-rmcat-transport-wide-cc-extensions-01 */
+	AST_RTP_EXTENSION_TRANSPORT_WIDE_CC,
 	/*! The maximum number of known RTP extensions */
 	AST_RTP_EXTENSION_MAX,
 };
@@ -585,6 +590,26 @@ struct ast_rtp_engine_dtls {
 	const char *(*get_fingerprint)(struct ast_rtp_instance *instance);
 };
 
+#ifdef TEST_FRAMEWORK
+/*! \brief Structure that represents the test functionality for res_rtp_asterisk unit tests */
+struct ast_rtp_engine_test {
+	/*! Drops RTP packets while this has a value greater than 0 */
+	int packets_to_drop;
+	/*! Sends a SR/RR instead of RTP the next time RTP would be sent */
+	int send_report;
+	/*! Set to 1 whenever SDES is received */
+	int sdes_received;
+	/*! Get the number of packets in the receive buffer for a RTP instance */
+	size_t (*recv_buffer_count)(struct ast_rtp_instance *instance);
+	/*! Get the maximum number of packets the receive buffer can hold for a RTP instance */
+	size_t (*recv_buffer_max)(struct ast_rtp_instance *instance);
+	/*! Get the number of packets in the send buffer for a RTP instance */
+	size_t (*send_buffer_count)(struct ast_rtp_instance *instance);
+	/*! Set the schedid for RTCP */
+	void (*set_schedid)(struct ast_rtp_instance *instance, int id);
+};
+#endif
+
 /*! Structure that represents an RTP stack (engine) */
 struct ast_rtp_engine {
 	/*! Name of the RTP engine, used when explicitly requested */
@@ -666,6 +691,10 @@ struct ast_rtp_engine {
 	struct ast_rtp_engine_ice *ice;
 	/*! Callback to pointer for optional DTLS SRTP support */
 	struct ast_rtp_engine_dtls *dtls;
+#ifdef TEST_FRAMEWORK
+	/*! Callback to pointer for test callbacks for RTP/RTCP unit tests */
+	struct ast_rtp_engine_test *test;
+#endif
 	/*! Callback to enable an RTP extension (returns non-zero if supported) */
 	int (*extension_enable)(struct ast_rtp_instance *instance, enum ast_rtp_extension extension);
 	/*! Linked list information */
@@ -754,7 +783,7 @@ enum ast_rtp_extension_direction {
  * \brief Allocation routine for \ref ast_rtp_payload_type
  *
  * \retval NULL on error
- * \retval An ao2 ref counted \c ast_rtp_payload_type on success.
+ * \return An ao2 ref counted \c ast_rtp_payload_type on success.
  *
  * \note The \c ast_rtp_payload_type returned by this function is an
  *       ao2 ref counted object.
@@ -1245,7 +1274,7 @@ void ast_rtp_instance_set_prop(struct ast_rtp_instance *instance, enum ast_rtp_p
  * \param instance The RTP instance to get the property from
  * \param property The property to get
  *
- * \retval Current value of the property
+ * \return Current value of the property
  *
  * Example usage:
  *
@@ -1346,7 +1375,7 @@ size_t ast_rtp_instance_extmap_count(struct ast_rtp_instance *instance);
  * \param instance The RTP instance to retrieve the extension from
  * \param id The negotiated RTP extension id
  *
- * \retval extension the extension that maps to the id
+ * \return extension the extension that maps to the id
  *
  * \since 15.5.0
  *
@@ -1361,7 +1390,7 @@ enum ast_rtp_extension ast_rtp_instance_extmap_get_extension(struct ast_rtp_inst
  * \param instance The RTP instance to retrieve the direction from
  * \param id The negotiated RTP extension id
  *
- * \retval direction the direction that has been negotiated
+ * \return direction the direction that has been negotiated
  *
  * \since 15.5.0
  */
@@ -1373,7 +1402,7 @@ enum ast_rtp_extension_direction ast_rtp_instance_extmap_get_direction(struct as
  * \param instance The RTP instance to retrieve the direction from
  * \param id The negotiated RTP extension id
  *
- * \retval uri The URI for the RTP extension
+ * \return The URI for the RTP extension
  *
  * \since 15.5.0
  */
@@ -1459,8 +1488,6 @@ void ast_rtp_codecs_payloads_copy(struct ast_rtp_codecs *src, struct ast_rtp_cod
  * \param src The source codecs structure
  * \param dest The destination codecs structure that the values from src will be copied to
  * \param instance Optionally the instance that the dst codecs structure belongs to
- *
- * \return Nothing
  */
 void ast_rtp_codecs_payloads_xover(struct ast_rtp_codecs *src, struct ast_rtp_codecs *dest, struct ast_rtp_instance *instance);
 
@@ -1490,7 +1517,7 @@ void ast_rtp_codecs_payloads_set_m_type(struct ast_rtp_codecs *codecs, struct as
  * \param instance Optionally the instance that the codecs structure belongs to
  * \param payload Numerical payload that was seen in the a=rtpmap: SDP line
  * \param mimetype The string mime type that was seen
- * \param mimesubtype The strin mime sub type that was seen
+ * \param mimesubtype The string mime sub type that was seen
  * \param options Optional options that may change the behavior of this specific payload
  *
  * \retval 0 success
@@ -1569,7 +1596,7 @@ enum ast_media_type ast_rtp_codecs_get_stream_type(struct ast_rtp_codecs *codecs
  * \param codecs Codecs structure to look in
  * \param payload Numerical payload to look up
  *
- * \retval Payload information.
+ * \return Payload information.
  * \retval NULL if payload does not exist.
  *
  * \note The payload returned by this function has its reference count increased.
@@ -1606,7 +1633,7 @@ int ast_rtp_codecs_payload_replace_format(struct ast_rtp_codecs *codecs, int pay
  * \param codecs Codecs structure to look in
  * \param payload Numerical payload type to look up
  *
- * \retval pointer to format structure on success
+ * \return pointer to format structure on success
  * \retval NULL on failure
  *
  * \note The format returned by this function has its reference count increased.
@@ -1631,7 +1658,7 @@ void ast_rtp_codecs_set_framing(struct ast_rtp_codecs *codecs, unsigned int fram
  *
  * \param codecs Codecs structure to get the framing from
  *
- * \retval The framing to be used for the media stream associated with these codecs
+ * \return The framing to be used for the media stream associated with these codecs
  *
  * \since 13.0.0
  */
@@ -1686,7 +1713,7 @@ void ast_rtp_codecs_payload_formats(struct ast_rtp_codecs *codecs, struct ast_fo
  * is an Asterisk format or non-format code.  If one is currently not
  * assigned then create a rx payload type mapping.
  *
- * \retval Numerical payload type
+ * \return Numerical payload type
  * \retval -1 if could not assign.
  *
  * Example usage:
@@ -1724,7 +1751,7 @@ int ast_rtp_codecs_payload_set_rx(struct ast_rtp_codecs *codecs, int code, struc
  * \param format Asterisk format to look for
  * \param code The format to look for
  *
- * \retval Numerical payload type
+ * \return Numerical payload type
  * \retval -1 if not found.
  */
 int ast_rtp_codecs_payload_code_tx(struct ast_rtp_codecs *codecs, int asterisk_format, const struct ast_format *format, int code);
@@ -1735,7 +1762,7 @@ int ast_rtp_codecs_payload_code_tx(struct ast_rtp_codecs *codecs, int asterisk_f
  * \param codecs Codecs structure to look in
  * \param payload The payload type format to look for
  *
- * \retval Numerical payload type or -1 if unable to find payload in codecs
+ * \return Numerical payload type or -1 if unable to find payload in codecs
  *
  * Example usage:
  *
@@ -1755,7 +1782,7 @@ int ast_rtp_codecs_find_payload_code(struct ast_rtp_codecs *codecs, int payload)
  * \param code RTP code to look up
  * \param options Additional options that may change the result
  *
- * \retval Mime subtype success
+ * \return Mime subtype success
  * \retval NULL failure
  *
  * Example usage:
@@ -1871,7 +1898,7 @@ int ast_rtp_instance_dtmf_mode_set(struct ast_rtp_instance *instance, enum ast_r
  *
  * \param instance The RTP instance to get the DTMF mode of
  *
- * \retval DTMF mode
+ * \return DTMF mode
  *
  * Example usage:
  *
@@ -2011,7 +2038,7 @@ struct ast_rtp_glue *ast_rtp_instance_get_glue(const char *type);
  *
  * \param instance The RTP instance
  *
- * \retval The unique ID of the channel
+ * \return The unique ID of the channel
  * \retval Empty string if no channel owns this RTP instance
  *
  * \since 12
@@ -2364,7 +2391,7 @@ void ast_rtp_instance_set_keepalive(struct ast_rtp_instance *instance, int timeo
  *
  * \param instance The RTP instance
  *
- * \retval timeout value
+ * \return timeout value
  *
  * Example usage:
  *
@@ -2383,7 +2410,7 @@ int ast_rtp_instance_get_timeout(struct ast_rtp_instance *instance);
  *
  * \param instance The RTP instance
  *
- * \retval timeout value
+ * \return timeout value
  *
  * Example usage:
  *
@@ -2402,7 +2429,7 @@ int ast_rtp_instance_get_hold_timeout(struct ast_rtp_instance *instance);
  *
  * \param instance The RTP instance
  *
- * \retval period Keepalive interval value
+ * \return period Keepalive interval value
  *
  * Example usage:
  *
@@ -2421,7 +2448,7 @@ int ast_rtp_instance_get_keepalive(struct ast_rtp_instance *instance);
  *
  * \param instance The RTP instance
  *
- * \retval pointer to the engine
+ * \return pointer to the engine
  *
  * Example usage:
  *
@@ -2440,7 +2467,7 @@ struct ast_rtp_engine *ast_rtp_instance_get_engine(struct ast_rtp_instance *inst
  *
  * \param instance The RTP instance
  *
- * \retval pointer to the glue
+ * \return pointer to the glue
  *
  * Example:
  *
@@ -2485,7 +2512,7 @@ int ast_rtp_instance_add_srtp_policy(struct ast_rtp_instance *instance, struct a
  *
  * \param instance the RTP instance
  * \param rtcp 1 to request instance for RTCP
- * \retval the SRTP instance on success
+ * \return the SRTP instance on success
  * \retval NULL if no SRTP instance exists
  */
 struct ast_srtp *ast_rtp_instance_get_srtp(struct ast_rtp_instance *instance, int rtcp);
@@ -2504,17 +2531,29 @@ int ast_rtp_engine_unload_format(struct ast_format *format);
  *
  * \param instance the RTP instance
  *
- * \retval ICE support if present
+ * \return ICE support if present
  * \retval NULL if no ICE support available
  */
 struct ast_rtp_engine_ice *ast_rtp_instance_get_ice(struct ast_rtp_instance *instance);
 
+#ifdef TEST_FRAMEWORK
+/*!
+ * \brief Obtain a pointer to the test callbacks on an RTP instance
+ *
+ * \param instance the RTP instance
+ *
+ * \return test callbacks if present
+ * \retval NULL if not present
+ */
+struct ast_rtp_engine_test *ast_rtp_instance_get_test(struct ast_rtp_instance *instance);
+#endif
+
 /*!
  * \brief Obtain a pointer to the DTLS support present on an RTP instance
  *
  * \param instance the RTP instance
  *
- * \retval DTLS support if present
+ * \return DTLS support if present
  * \retval NULL if no DTLS support available
  */
 struct ast_rtp_engine_dtls *ast_rtp_instance_get_dtls(struct ast_rtp_instance *instance);
@@ -2562,7 +2601,7 @@ struct ast_json;
  * \brief Allocate an ao2 ref counted instance of \ref ast_rtp_rtcp_report
  *
  * \param report_blocks The number of report blocks to allocate
- * \retval An ao2 ref counted \ref ast_rtp_rtcp_report object on success
+ * \return An ao2 ref counted \ref ast_rtp_rtcp_report object on success
  * \retval NULL on error
  */
 struct ast_rtp_rtcp_report *ast_rtp_rtcp_report_alloc(unsigned int report_blocks);
@@ -2598,7 +2637,7 @@ time_t ast_rtp_instance_get_last_tx(const struct ast_rtp_instance *rtp);
  */
 void ast_rtp_instance_set_last_tx(struct ast_rtp_instance *rtp, time_t time);
 
-/*
+/*!
  * \brief Get the last RTP reception time
  *
  * \param rtp The instance from which to get the last reception time
@@ -2657,7 +2696,7 @@ void ast_rtp_instance_set_remote_ssrc(struct ast_rtp_instance *rtp, unsigned int
  * \brief Set the stream number for an RTP instance
  * \since 15.0.0
  *
- * \param rtp The RTP instance
+ * \param instance The RTP instance
  * \param stream_num The stream identifier number
  */
 void ast_rtp_instance_set_stream_num(struct ast_rtp_instance *instance, int stream_num);
@@ -2670,7 +2709,7 @@ void ast_rtp_instance_set_stream_num(struct ast_rtp_instance *instance, int stre
  * \since 12
  * \brief Message type for an RTCP message sent from this Asterisk instance
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_rtp_rtcp_sent_type(void);
 
@@ -2678,19 +2717,186 @@ struct stasis_message_type *ast_rtp_rtcp_sent_type(void);
  * \since 12
  * \brief Message type for an RTCP message received from some external source
  *
- * \retval A stasis message type
+ * \return A stasis message type
  */
 struct stasis_message_type *ast_rtp_rtcp_received_type(void);
 
+/*! @} */
+
+#ifdef TEST_FRAMEWORK
+/*!
+ * \brief Get the maximum size of the receive buffer
+ *
+ * \param instance The RTP instance
+ * \return The recv_buffer max size if it exists, else 0
+ */
+size_t ast_rtp_instance_get_recv_buffer_max(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Get the current size of the receive buffer
+ *
+ * \param instance The RTP instance
+ * \return The recv_buffer size if it exists, else 0
+ */
+size_t ast_rtp_instance_get_recv_buffer_count(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Get the current size of the send buffer
+ *
+ * \param instance The RTP instance
+ * \return The send_buffer size if it exists, else 0
+ */
+size_t ast_rtp_instance_get_send_buffer_count(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Set the schedid for RTCP
+ *
+ * \param instance The RTP instance
+ * \param id The number to set schedid to
+ */
+void ast_rtp_instance_set_schedid(struct ast_rtp_instance *instance, int id);
+
+/*!
+ * \brief Set the number of packets to drop on RTP read
+ *
+ * \param instance The RTP instance
+ * \param num The number of packets to drop
+ */
+void ast_rtp_instance_drop_packets(struct ast_rtp_instance *instance, int num);
+
+/*!
+ * \brief Sends a SR/RR report the next time RTP would be sent
+ *
+ * \param instance The RTP instance
+ */
+void ast_rtp_instance_queue_report(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Get the value of sdes_received on the test engine
+ *
+ * \param instance The RTP instance
+ * \retval 1 if sdes_received, else 0
+ */
+int ast_rtp_instance_get_sdes_received(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Resets all the fields to default values for the test engine
+ *
+ * \param instance The RTP instance
+ */
+void ast_rtp_instance_reset_test_engine(struct ast_rtp_instance *instance);
+#endif
+
+/*!
+ * \brief Convert given stat instance into json format
+ * \param stats
+ * \return A json format stat
+ */
+struct ast_json *ast_rtp_convert_stats_json(const struct ast_rtp_instance_stats *stats);
+
+/*!
+ * \brief Retrieve statistics about an RTP instance in json format
+ * \param instance
+ * \return json object of stats
+ */
+struct ast_json *ast_rtp_instance_get_stats_all_json(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Retrieve the sample rate of a format according to RTP specifications
+ * \since 16.7.0
+ * \since 17.1.0
+ *
+ * \param format The media format
+ *
+ * \return The sample rate
+ */
+int ast_rtp_get_rate(const struct ast_format *format);
+
 /*!
  * \since 12
  * \brief \ref stasis topic for RTP and RTCP related messages
  *
- * \retval A \ref stasis topic
+ * \return A \ref stasis topic
  */
 struct stasis_topic *ast_rtp_topic(void);
 
-/* @} */
+/* RTP debug logging category name */
+#define AST_LOG_CATEGORY_RTP "rtp"
+/* RTP packet debug logging category name */
+#define AST_LOG_CATEGORY_RTP_PACKET "rtp_packet"
+/* RTCP debug logging category name */
+#define AST_LOG_CATEGORY_RTCP "rtcp"
+/* RTCP packet debug logging category name */
+#define AST_LOG_CATEGORY_RTCP_PACKET "rtcp_packet"
+/* DTLS debug logging category name */
+#define AST_LOG_CATEGORY_DTLS "dtls"
+/* DTLS packet debug logging category name */
+#define AST_LOG_CATEGORY_DTLS_PACKET "dtls_packet"
+/* ICE debug logging category name */
+#define AST_LOG_CATEGORY_ICE "ice"
+
+uintmax_t ast_debug_category_rtp_id(void);
+uintmax_t ast_debug_category_rtp_packet_id(void);
+uintmax_t ast_debug_category_rtcp_id(void);
+uintmax_t ast_debug_category_rtcp_packet_id(void);
+uintmax_t ast_debug_category_dtls_id(void);
+uintmax_t ast_debug_category_dtls_packet_id(void);
+uintmax_t ast_debug_category_ice_id(void);
+
+#define AST_DEBUG_CATEGORY_RTP ast_debug_category_rtp_id() /* RTP debug logging category id */
+#define AST_DEBUG_CATEGORY_RTP_PACKET ast_debug_category_rtp_packet_id() /* RTP packet debug logging category id */
+#define AST_DEBUG_CATEGORY_RTCP ast_debug_category_rtcp_id() /* RTCP debug logging category id */
+#define AST_DEBUG_CATEGORY_RTCP_PACKET ast_debug_category_rtcp_packet_id() /* RTCP packet debug logging category id */
+#define AST_DEBUG_CATEGORY_DTLS ast_debug_category_dtls_id() /* DTLS debug logging category id */
+#define AST_DEBUG_CATEGORY_DTLS_PACKET ast_debug_category_dtls_packet_id() /* DTLS packet debug logging category id */
+#define AST_DEBUG_CATEGORY_ICE ast_debug_category_ice_id() /* ICE debug logging category id */
+
+/*!
+ * \brief Log debug level RTP information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_rtp(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_RTP,  __VA_ARGS__)
+
+/* Allow logging of RTP packets? */
+#define ast_debug_rtp_packet_is_allowed \
+	ast_debug_category_is_allowed(AST_LOG_CATEGORY_ENABLED, AST_DEBUG_CATEGORY_RTP_PACKET)
+
+/*!
+ * \brief Log debug level RTCP information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_rtcp(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_RTCP, __VA_ARGS__)
+
+/* Allow logging of RTCP packets? */
+#define ast_debug_rtcp_packet_is_allowed \
+	ast_debug_category_is_allowed(AST_LOG_CATEGORY_ENABLED, AST_DEBUG_CATEGORY_RTCP_PACKET)
+
+/*!
+ * \brief Log debug level DTLS information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_dtls(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_DTLS, __VA_ARGS__)
+
+/* Allow logging of DTLS packets? */
+#define ast_debug_dtls_packet_is_allowed \
+	ast_debug_category_is_allowed(AST_LOG_CATEGORY_ENABLED, AST_DEBUG_CATEGORY_DTLS_PACKET)
+/*!
+ * \brief Log debug level ICE information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_ice(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_ICE, __VA_ARGS__)
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
