@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file taskprocessor.h
+ * \file
+ *
  * \brief An API for managing task processing threads that can be shared across modules
  *
  * \author Dwayne M. Hubbard <dhubbard@digium.com>
@@ -57,7 +58,7 @@
 struct ast_taskprocessor;
 
 /*! \brief Suggested maximum taskprocessor name length (less null terminator). */
-#define AST_TASKPROCESSOR_MAX_NAME	45
+#define AST_TASKPROCESSOR_MAX_NAME	70
 
 /*! Default taskprocessor high water level alert trigger */
 #define AST_TASKPROCESSOR_HIGH_WATER_LEVEL 500
@@ -177,7 +178,7 @@ struct ast_taskprocessor *ast_taskprocessor_get(const char *name, enum ast_tps_o
  * \param name The name of the taskprocessor to create
  * \param listener The listener for operations on this taskprocessor
  * \retval NULL Failure
- * \reval non-NULL success
+ * \retval non-NULL success
  */
 struct ast_taskprocessor *ast_taskprocessor_create_with_listener(const char *name, struct ast_taskprocessor_listener *listener);
 
@@ -304,6 +305,15 @@ int ast_taskprocessor_is_task(struct ast_taskprocessor *tps);
  */
 unsigned int ast_taskprocessor_seq_num(void);
 
+/*!
+ * \brief Append the next sequence number to the given string, and copy into the buffer.
+ *
+ * \param buf Where to copy the appended taskprocessor name.
+ * \param size How large is buf including null terminator.
+ * \param name A name to append the sequence number to.
+ */
+void ast_taskprocessor_name_append(char *buf, unsigned int size, const char *name);
+
 /*!
  * \brief Build a taskprocessor name with a sequence number on the end.
  * \since 13.8.0
@@ -315,8 +325,6 @@ unsigned int ast_taskprocessor_seq_num(void);
  * \note The user supplied part of the taskprocessor name is truncated
  * to allow the full sequence number to be appended within the supplied
  * buffer size.
- *
- * \return Nothing
  */
 void __attribute__((format(printf, 3, 4))) ast_taskprocessor_build_name(char *buf, unsigned int size, const char *format, ...);
 
@@ -341,6 +349,19 @@ long ast_taskprocessor_size(struct ast_taskprocessor *tps);
  */
 unsigned int ast_taskprocessor_alert_get(void);
 
+
+/*!
+ * \brief Get the current taskprocessor high water alert count by subsystem.
+ * \since 13.26.0
+ * \since 16.3.0
+ *
+ * \param subsystem The subsystem name
+ *
+ * \retval 0 if no taskprocessors are in high water alert.
+ * \retval non-zero if some task processors are in high water alert.
+ */
+unsigned int ast_taskprocessor_get_subsystem_alert(const char *subsystem);
+
 /*!
  * \brief Set the high and low alert water marks of the given taskprocessor queue.
  * \since 13.10.0
