@@ -1389,8 +1389,6 @@ static char *aoc_cli_debug_enable(struct ast_cli_entry *e, int cmd, struct ast_c
  * \param name Name of the time structure to convert.
  * \param time Data to convert.
  * \param scale Data to convert.
- *
- * \return Nothing
  */
 static void aoc_time_str(struct ast_str **msg, const char *prefix, const char *name, unsigned long time, enum ast_aoc_time_scale scale)
 {
@@ -1407,9 +1405,7 @@ static void aoc_time_str(struct ast_str **msg, const char *prefix, const char *n
  * \param msg Event message string being built.
  * \param prefix Prefix to add to the amount lines.
  * \param amount Data to convert.
- * \param multipler to convert
- *
- * \return Nothing
+ * \param mult to convert
  */
 static void aoc_amount_str(struct ast_str **msg, const char *prefix, unsigned int amount, enum ast_aoc_currency_multiplier mult)
 {
@@ -1738,13 +1734,13 @@ static struct ast_json *s_to_json(const struct ast_aoc_decoded *decoded)
 				decoded->aoc_s_entries[i].rate.duration.amount,
 				decoded->aoc_s_entries[i].rate.duration.multiplier);
 
-			time = ast_json_pack("{s:i, s:i}",
-				"Length", decoded->aoc_s_entries[i].rate.duration.time,
+			time = ast_json_pack("{s:I, s:i}",
+				"Length", (ast_json_int_t)decoded->aoc_s_entries[i].rate.duration.time,
 				"Scale", decoded->aoc_s_entries[i].rate.duration.time_scale);
 
 			if (decoded->aoc_s_entries[i].rate.duration.granularity_time) {
-				granularity = ast_json_pack("{s:i, s:i}",
-					"Length", decoded->aoc_s_entries[i].rate.duration.granularity_time,
+				granularity = ast_json_pack("{s:I, s:i}",
+					"Length", (ast_json_int_t)decoded->aoc_s_entries[i].rate.duration.granularity_time,
 					"Scale", decoded->aoc_s_entries[i].rate.duration.granularity_time_scale);
 			}
 
@@ -1829,8 +1825,6 @@ static void aoc_event_blob_dtor(void *obj)
  * \param chan Channel associated with the AOC event. (May be NULL if no channel)
  * \param msg_type What kind of AOC event.
  * \param blob AOC data blob to publish.
- *
- * \return Nothing
  */
 static void aoc_publish_blob(struct ast_channel *chan, struct stasis_message_type *msg_type, struct ast_json *blob)
 {
