@@ -207,6 +207,8 @@ struct ast_channel {
 
 	char context[AST_MAX_CONTEXT];			/*!< Dialplan: Current extension context */
 	char exten[AST_MAX_EXTENSION];			/*!< Dialplan: Current extension number */
+	char lastcontext[AST_MAX_CONTEXT];		/*!< Dialplan: Previous extension context */
+	char lastexten[AST_MAX_EXTENSION];		/*!< Dialplan: Previous extension number */
 	char macrocontext[AST_MAX_CONTEXT];		/*!< Macro: Current non-macro context. See app_macro.c */
 	char macroexten[AST_MAX_EXTENSION];		/*!< Macro: Current non-macro extension. See app_macro.c */
 	char unbridged;							/*!< non-zero if the bridge core needs to re-evaluate the current
@@ -320,16 +322,32 @@ const char *ast_channel_context(const struct ast_channel *chan)
 {
 	return chan->context;
 }
+const char *ast_channel_lastcontext(const struct ast_channel *chan)
+{
+	return chan->lastcontext;
+}
 void ast_channel_context_set(struct ast_channel *chan, const char *value)
 {
+	if (!*chan->lastcontext || strcmp(value, chan->context)) {
+		/* only copy to last context when it changes, unless it's empty to begin with */
+		ast_copy_string(chan->lastcontext, chan->context, sizeof(chan->lastcontext));
+	}
 	ast_copy_string(chan->context, value, sizeof(chan->context));
 }
 const char *ast_channel_exten(const struct ast_channel *chan)
 {
 	return chan->exten;
 }
+const char *ast_channel_lastexten(const struct ast_channel *chan)
+{
+	return chan->lastexten;
+}
 void ast_channel_exten_set(struct ast_channel *chan, const char *value)
 {
+	if (!*chan->lastexten || strcmp(value, chan->exten)) {
+		/* only copy to last exten when it changes, unless it's empty to begin with */
+		ast_copy_string(chan->lastexten, chan->exten, sizeof(chan->lastexten));
+	}
 	ast_copy_string(chan->exten, value, sizeof(chan->exten));
 }
 const char *ast_channel_macrocontext(const struct ast_channel *chan)
@@ -1269,14 +1287,14 @@ void ast_channel_dialed_causes_clear(const struct ast_channel *chan)
 	ao2_callback(chan->dialed_causes, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL);
 }
 
-/* \brief Hash function for pvt cause code frames */
+/*! \brief Hash function for pvt cause code frames */
 static int pvt_cause_hash_fn(const void *vpc, const int flags)
 {
 	const struct ast_control_pvt_cause_code *pc = vpc;
 	return ast_str_hash(ast_tech_to_upper(ast_strdupa(pc->chan_name)));
 }
 
-/* \brief Comparison function for pvt cause code frames */
+/*! \brief Comparison function for pvt cause code frames */
 static int pvt_cause_cmp_fn(void *obj, void *vstr, int flags)
 {
 	struct ast_control_pvt_cause_code *pc = obj;
@@ -1480,15 +1498,24 @@ int ast_channel_forward_endpoint(struct ast_channel *chan,
 
 int ast_channel_internal_setup_topics(struct ast_channel *chan)
 {
-	const char *topic_name = chan->uniqueid.unique_id;
+	char *topic_name;
+	int ret;
 	ast_assert(chan->topics == NULL);
 
-	if (ast_strlen_zero(topic_name)) {
-		topic_name = "<dummy-channel>";
+	if (ast_strlen_zero(chan->uniqueid.unique_id)) {
+		static int dummy_id;
+		ret = ast_asprintf(&topic_name, "channel:dummy-%d", ast_atomic_fetchadd_int(&dummy_id, +1));
+	} else {
+		ret = ast_asprintf(&topic_name, "channel:%s", chan->uniqueid.unique_id);
+	}
+
+	if (ret < 0) {
+		return -1;
 	}
 
 	chan->topics = stasis_cp_single_create(
 		ast_channel_cache_all(), topic_name);
+	ast_free(topic_name);
 	if (!chan->topics) {
 		return -1;
 	}
@@ -1578,5 +1605,5 @@ void ast_channel_internal_swap_stream_topology(struct ast_channel *chan1,
 
 int ast_channel_is_multistream(struct ast_channel *chan)
 {
-	return (chan->tech && chan->tech->read_stream && chan->tech->write_stream);
+	return (chan && chan->tech && chan->tech->read_stream && chan->tech->write_stream);
 }
