@@ -457,6 +457,7 @@ static struct ast_codec g729a = {
 	.samples_count = g729_samples,
 	.get_length = g729_length,
 	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_G729,
 };
 
 static unsigned char get_n_bits_at(unsigned char *data, int n, int bit)
@@ -806,6 +807,13 @@ static struct ast_codec h264 = {
 	.sample_rate = 1000,
 };
 
+static struct ast_codec h265 = {
+	.name = "h265",
+	.description = "H.265 video",
+	.type = AST_MEDIA_TYPE_VIDEO,
+	.sample_rate = 1000,
+};
+
 static struct ast_codec mpeg4 = {
 	.name = "mpeg4",
 	.description = "MPEG4 video",
@@ -971,6 +979,7 @@ int ast_codec_builtin_init(void)
 	res |= CODEC_REGISTER_AND_CACHE(h263);
 	res |= CODEC_REGISTER_AND_CACHE(h263p);
 	res |= CODEC_REGISTER_AND_CACHE(h264);
+	res |= CODEC_REGISTER_AND_CACHE(h265);
 	res |= CODEC_REGISTER_AND_CACHE(mpeg4);
 	res |= CODEC_REGISTER_AND_CACHE(vp8);
 	res |= CODEC_REGISTER_AND_CACHE(vp9);
