@@ -166,14 +166,12 @@ static int predial_disable(void *data)
 static void answer_exec_run(struct ast_dial *dial, struct ast_dial_channel *dial_channel, char *app, char *args)
 {
 	struct ast_channel *chan = dial_channel->owner;
-	struct ast_app *ast_app = pbx_findapp(app);
 
-	/* If the application was not found, return immediately */
-	if (!ast_app)
+	/* Execute the application, if available */
+	if (ast_pbx_exec_application(chan, app, args)) {
+		/* If the application was not found, return immediately */
 		return;
-
-	/* All is well... execute the application */
-	pbx_exec(chan, ast_app, args);
+	}
 
 	/* If another thread is not taking over hang up the channel */
 	ast_mutex_lock(&dial->lock);
@@ -1083,11 +1081,6 @@ void ast_dial_hangup(struct ast_dial *dial)
 	return;
 }
 
-/*! \brief Destroys a dialing structure
- * \note Destroys (free's) the given ast_dial structure
- * \param dial Dialing structure to free
- * \return Returns 0 on success, -1 on failure
- */
 int ast_dial_destroy(struct ast_dial *dial)
 {
 	int i = 0;
@@ -1142,12 +1135,6 @@ int ast_dial_destroy(struct ast_dial *dial)
 	return 0;
 }
 
-/*! \brief Enables an option globally
- * \param dial Dial structure to enable option on
- * \param option Option to enable
- * \param data Data to pass to this option (not always needed)
- * \return Returns 0 on success, -1 on failure
- */
 int ast_dial_option_global_enable(struct ast_dial *dial, enum ast_dial_option option, void *data)
 {
 	/* If the option is already enabled, return failure */
@@ -1184,13 +1171,6 @@ static struct ast_dial_channel *find_dial_channel(struct ast_dial *dial, int num
 	return channel;
 }
 
-/*! \brief Enables an option per channel
- * \param dial Dial structure
- * \param num Channel number to enable option on
- * \param option Option to enable
- * \param data Data to pass to this option (not always needed)
- * \return Returns 0 on success, -1 on failure
- */
 int ast_dial_option_enable(struct ast_dial *dial, int num, enum ast_dial_option option, void *data)
 {
 	struct ast_dial_channel *channel = NULL;
@@ -1215,11 +1195,6 @@ int ast_dial_option_enable(struct ast_dial *dial, int num, enum ast_dial_option
 	return 0;
 }
 
-/*! \brief Disables an option globally
- * \param dial Dial structure to disable option on
- * \param option Option to disable
- * \return Returns 0 on success, -1 on failure
- */
 int ast_dial_option_global_disable(struct ast_dial *dial, enum ast_dial_option option)
 {
 	/* If the option is not enabled, return failure */
@@ -1237,12 +1212,6 @@ int ast_dial_option_global_disable(struct ast_dial *dial, enum ast_dial_option o
 	return 0;
 }
 
-/*! \brief Disables an option per channel
- * \param dial Dial structure
- * \param num Channel number to disable option on
- * \param option Option to disable
- * \return Returns 0 on success, -1 on failure
- */
 int ast_dial_option_disable(struct ast_dial *dial, int num, enum ast_dial_option option)
 {
 	struct ast_dial_channel *channel = NULL;
@@ -1305,11 +1274,6 @@ void *ast_dial_get_user_data(struct ast_dial *dial)
 	return dial->user_data;
 }
 
-/*! \brief Set the maximum time (globally) allowed for trying to ring phones
- * \param dial The dial structure to apply the time limit to
- * \param timeout Maximum time allowed
- * \return nothing
- */
 void ast_dial_set_global_timeout(struct ast_dial *dial, int timeout)
 {
 	dial->timeout = timeout;
@@ -1320,12 +1284,6 @@ void ast_dial_set_global_timeout(struct ast_dial *dial, int timeout)
 	return;
 }
 
-/*! \brief Set the maximum time (per channel) allowed for trying to ring the phone
- * \param dial The dial structure the channel belongs to
- * \param num Channel number to set timeout on
- * \param timeout Maximum time allowed
- * \return nothing
- */
 void ast_dial_set_timeout(struct ast_dial *dial, int num, int timeout)
 {
 	struct ast_dial_channel *channel = NULL;
