@@ -184,30 +184,34 @@
 					</description>
 				</configOption>
 				<configOption name="automon">
-					<synopsis>DTMF sequence to start or stop monitoring a call</synopsis>
+					<synopsis>DTMF sequence to start or stop Monitor on a call</synopsis>
 					<description>
 						<para>This will cause the channel that pressed the DTMF sequence
 						to be monitored by the <literal>Monitor</literal> application. The
 						format for the recording is determined by the <replaceable>TOUCH_MONITOR_FORMAT</replaceable>
 						channel variable. If this variable is not specified, then <literal>wav</literal> is the
 						default. The filename is constructed in the following manner:</para>
-
-						<para>    prefix-timestamp-filename</para>
-
+						<para>    prefix-timestamp-suffix.fmt</para>
 						<para>where prefix is either the value of the <replaceable>TOUCH_MONITOR_PREFIX</replaceable>
 						channel variable or <literal>auto</literal> if the variable is not set. The timestamp
-						is a UNIX timestamp. The filename is either the value of the <replaceable>TOUCH_MONITOR</replaceable>
+						is a UNIX timestamp. The suffix is either the value of the <replaceable>TOUCH_MONITOR</replaceable>
 						channel variable or the callerID of the channels if the variable is not set.</para>
 					</description>
+					<see-also><ref type="configOption">automixmon</ref></see-also>
 				</configOption>
 				<configOption name="automixmon">
-					<synopsis>DTMF sequence to start or stop mixmonitoring a call </synopsis>
+					<synopsis>DTMF sequence to start or stop MixMonitor on a call</synopsis>
 					<description>
-						<para>Operation of the automixmon is similar to the <literal> automon </literal>
-						feature, with the following exceptions:
-							<replaceable>TOUCH_MIXMONITOR</replaceable> is used in place of <replaceable>TOUCH_MONITOR</replaceable>
-							<replaceable>TOUCH_MIXMONITOR_FORMAT</replaceable> is used in place of <replaceable>TOUCH_MIXMONITOR</replaceable>
-							There is no equivalent for <replaceable>TOUCH_MONITOR_PREFIX</replaceable>. <literal>"auto"</literal> is always how the filename begins.</para>
+						<para>This will cause the channel that pressed the DTMF sequence
+						to be monitored by the <literal>MixMonitor</literal> application. The
+						format for the recording is determined by the <replaceable>TOUCH_MIXMONITOR_FORMAT</replaceable>
+						channel variable. If this variable is not specified, then <literal>wav</literal> is the
+						default. The filename is constructed in the following manner:</para>
+						<para>    prefix-timestamp-suffix.fmt</para>
+						<para>where prefix is either the value of the <replaceable>TOUCH_MIXMONITOR_PREFIX</replaceable>
+						channel variable or <literal>auto</literal> if the variable is not set. The timestamp
+						is a UNIX timestamp. The suffix is either the value of the <replaceable>TOUCH_MIXMONITOR</replaceable>
+						channel variable or the callerID of the channels if the variable is not set.</para>
 					</description>
 					<see-also><ref type="configOption">automon</ref></see-also>
 				</configOption>
@@ -726,7 +730,7 @@ static struct ao2_container *applicationmap_alloc(int replace_duplicates)
  * need to allocate these structures because they are not used.
  *
  * \param allocate_applicationmap See previous explanation
- * \retval NULL Failed to alloate configuration
+ * \retval NULL Failed to allocate configuration
  * \retval non-NULL Allocated configuration
  */
 static struct features_config *__features_config_alloc(int allocate_applicationmap)
@@ -817,7 +821,7 @@ static void features_copy(struct features_config *dest, const struct features_co
 
 	/* applicationmap and featuregroups are purposely not copied. A channel's applicationmap
 	 * is produced on the fly when ast_get_chan_applicationmap() is called
-	 * NOTE: This does not apply to the global cfg->applicationmap and cfg->featuresgroups
+	 * NOTE: This does not apply to the global cfg->applicationmap and cfg->featuregroups
 	 */
 }
 
