@@ -184,6 +184,41 @@ int ast_format_def_unregister(const char *name)
 	return res;
 }
 
+FILE *ast_file_mkftemp(char *template, mode_t mode)
+{
+	FILE *p = NULL;
+	int pfd = mkstemp(template);
+	chmod(template, mode);
+	if (pfd > -1) {
+		p = fdopen(pfd, "w+");
+		if (!p) {
+			close(pfd);
+			pfd = -1;
+		}
+	}
+	return p;
+}
+
+int ast_file_fdtemp(const char *path, char **filename, const char *template_name)
+{
+	int fd;
+
+	if (ast_asprintf(filename, "%s/%s", path, template_name) < 0) {
+		ast_log(LOG_ERROR, "Failed to set up temporary file path\n");
+		return -1;
+	}
+
+	ast_mkdir(path, 0644);
+
+	if ((fd = mkstemp(*filename)) < 0) {
+		ast_log(LOG_NOTICE, "Failed to create temporary file\n");
+		ast_free(*filename);
+		return -1;
+	}
+
+	return fd;
+}
+
 int ast_stopstream(struct ast_channel *tmp)
 {
 	ast_channel_lock(tmp);
@@ -271,7 +306,7 @@ int ast_writestream(struct ast_filestream *fs, struct ast_frame *f)
 static int copy(const char *infile, const char *outfile)
 {
 	int ifd, ofd, len;
-	char buf[4096];	/* XXX make it lerger. */
+	char buf[4096];	/* XXX make it larger. */
 
 	if ((ifd = open(infile, O_RDONLY)) < 0) {
 		ast_log(LOG_WARNING, "Unable to open %s in read-only mode\n", infile);
@@ -315,6 +350,8 @@ static char *build_filename(const char *filename, const char *ext)
 {
 	char *fn = NULL;
 
+	/* The wav49 -> WAV translation is duplicated in apps/app_mixmonitor.c, so
+	   if you change it here you need to change it there as well */
 	if (!strcmp(ext, "wav49"))
 		ext = "WAV";
 
@@ -517,7 +554,9 @@ static int filehelper(const char *filename, const void *arg2, const char *fmt, c
 	AST_RWLIST_RDLOCK(&formats);
 	/* Check for a specific format */
 	AST_RWLIST_TRAVERSE(&formats, f, list) {
-		char *stringp, *ext = NULL;
+		char *ext = NULL;
+		char storage[strlen(f->exts) + 1];
+		char *stringp;
 
 		if (fmt && !exts_compare(f->exts, fmt))
 			continue;
@@ -526,7 +565,8 @@ static int filehelper(const char *filename, const void *arg2, const char *fmt, c
 		 * The file must exist, and for OPEN, must match
 		 * one of the formats supported by the channel.
 		 */
-		stringp = ast_strdupa(f->exts);	/* this is in the stack so does not need to be freed */
+		strcpy(storage, f->exts); /* safe - this is in the stack so does not need to be freed */
+		stringp = storage;
 		while ( (ext = strsep(&stringp, "|")) ) {
 			struct stat st;
 			char *fn = build_filename(filename, ext);
@@ -637,11 +677,16 @@ static int is_absolute_path(const char *filename)
 	return filename[0] == '/';
 }
 
+static int is_remote_path(const char *filename)
+{
+	return strstr(filename, "://") ? 1 : 0;
+}
+
 /*!
  * \brief test if a file exists for a given format.
  * \note result_cap is OPTIONAL
- * \retval 1, true and result_cap represents format capabilities file exists in.
- * \retval 0, false
+ * \retval 1 true and result_cap represents format capabilities file exists in.
+ * \retval 0 false
  */
 static int fileexists_test(const char *filename, const char *fmt, const char *lang,
 			   char *buf, int buflen, struct ast_format_cap *result_cap)
@@ -650,7 +695,7 @@ static int fileexists_test(const char *filename, const char *fmt, const char *la
 		return 0;
 	}
 
-	if (!ast_media_cache_retrieve(filename, NULL, buf, buflen)) {
+	if (is_remote_path(filename) && !ast_media_cache_retrieve(filename, NULL, buf, buflen)) {
 		return filehelper(buf, result_cap, NULL, ACTION_EXISTS);
 	}
 
@@ -690,8 +735,8 @@ static int fileexists_test(const char *filename, const char *fmt, const char *la
  * \param result_cap OPTIONAL format capabilities result structure
  *        returns what formats the file was found in.
  *
- * \retval 1, true. file exists and result format is set
- * \retval 0, false. file does not exist.
+ * \retval 1 true. file exists and result format is set
+ * \retval 0 false. file does not exist.
  */
 static int fileexists_core(const char *filename, const char *fmt, const char *preflang,
 			   char *buf, int buflen, struct ast_format_cap *result_cap)
@@ -1052,6 +1097,12 @@ int ast_stream_fastforward(struct ast_filestream *fs, off_t ms)
 
 int ast_stream_rewind(struct ast_filestream *fs, off_t ms)
 {
+	off_t offset = ast_tellstream(fs);
+	if (ms * DEFAULT_SAMPLES_PER_MS > offset) {
+		/* Don't even bother asking the file format to seek to a negative offset... */
+		ast_debug(1, "Restarting, rather than seeking to negative offset %ld\n", (long) (offset - (ms * DEFAULT_SAMPLES_PER_MS)));
+		return ast_seekstream(fs, 0, SEEK_SET);
+	}
 	return ast_seekstream(fs, -ms * DEFAULT_SAMPLES_PER_MS, SEEK_CUR);
 }
 
@@ -1407,13 +1458,13 @@ struct ast_filestream *ast_writefile(const char *filename, const char *type, con
 			  We touch orig_fn just as a place-holder so other things (like vmail) see the file is there.
 			  What we are really doing is writing to record_cache_dir until we are done then we will mv the file into place.
 			*/
-			orig_fn = ast_strdupa(fn);
+			orig_fn = ast_strdup(fn);
 			for (c = fn; *c; c++)
 				if (*c == '/')
 					*c = '_';
 
 			size = strlen(fn) + strlen(record_cache_dir) + 2;
-			buf = ast_alloca(size);
+			buf = ast_malloc(size);
 			strcpy(buf, record_cache_dir);
 			strcat(buf, "/");
 			strcat(buf, fn);
@@ -1444,14 +1495,18 @@ struct ast_filestream *ast_writefile(const char *filename, const char *type, con
 				if (orig_fn) {
 					unlink(fn);
 					unlink(orig_fn);
+					ast_free(orig_fn);
 				}
 				if (fs) {
 					ast_closestream(fs);
 					fs = NULL;
 				}
-				if (!buf) {
-					ast_free(fn);
-				}
+				/*
+				 * 'fn' was has either been allocated from build_filename, or that was freed
+				 * and now 'fn' points to memory allocated for 'buf'. Either way the memory
+				 * now needs to be released.
+				 */
+				ast_free(fn);
 				continue;
 			}
 			fs->trans = NULL;
@@ -1459,8 +1514,14 @@ struct ast_filestream *ast_writefile(const char *filename, const char *type, con
 			fs->flags = flags;
 			fs->mode = mode;
 			if (orig_fn) {
-				fs->realfilename = ast_strdup(orig_fn);
-				fs->filename = ast_strdup(fn);
+				fs->realfilename = orig_fn;
+				fs->filename = fn;
+				/*
+				 * The above now manages the memory allocated for 'orig_fn' and 'fn', so
+				 * set them to NULL, so they don't get released at the end of the loop.
+				 */
+				orig_fn = NULL;
+				fn = NULL;
 			} else {
 				fs->realfilename = NULL;
 				fs->filename = ast_strdup(filename);
@@ -1473,9 +1534,9 @@ struct ast_filestream *ast_writefile(const char *filename, const char *type, con
 			if (orig_fn)
 				unlink(orig_fn);
 		}
-		/* if buf != NULL then fn is already free and pointing to it */
-		if (!buf)
-			ast_free(fn);
+		/* Free 'fn', or if 'fn' points to 'buf' then free 'buf' */
+		ast_free(fn);
+		ast_free(orig_fn);
 	}
 
 	AST_RWLIST_UNLOCK(&formats);
@@ -1682,6 +1743,7 @@ static int waitstream_core(struct ast_channel *c,
 				case AST_CONTROL_AOC:
 				case AST_CONTROL_UPDATE_RTP_PEER:
 				case AST_CONTROL_PVT_CAUSE_CODE:
+				case AST_CONTROL_FLASH:
 				case -1:
 					/* Unimportant */
 					break;
