@@ -37,6 +37,7 @@
 #include "asterisk/say.h"
 #include "asterisk/app.h"
 #include "asterisk/module.h"
+#include "asterisk/conversions.h"
 #include "pbx_private.h"
 
 /*** DOCUMENTATION
@@ -82,6 +83,9 @@
 						<para>Only break if a digit hit matches a one digit
 						extension in the destination context.</para>
 					</option>
+					<option name="p">
+						<para>Do not allow playback to be interrupted with digits.</para>
+					</option>
 				</optionlist>
 			</parameter>
 			<parameter name="langoverride">
@@ -440,9 +444,12 @@
 		</description>
 		<see-also>
 			<ref type="application">SayDigits</ref>
+			<ref type="application">SayMoney</ref>
 			<ref type="application">SayNumber</ref>
+			<ref type="application">SayOrdinal</ref>
 			<ref type="application">SayPhonetic</ref>
 			<ref type="function">CHANNEL</ref>
+			<ref type="function">SAYFILES</ref>
 		</see-also>
 	</application>
 	<application name="SayAlphaCase" language="en_US">
@@ -481,7 +488,9 @@
 		</description>
 		<see-also>
 			<ref type="application">SayDigits</ref>
+			<ref type="application">SayMoney</ref>
 			<ref type="application">SayNumber</ref>
+			<ref type="application">SayOrdinal</ref>
 			<ref type="application">SayPhonetic</ref>
 			<ref type="application">SayAlpha</ref>
 			<ref type="function">CHANNEL</ref>
@@ -503,9 +512,35 @@
 		</description>
 		<see-also>
 			<ref type="application">SayAlpha</ref>
+			<ref type="application">SayMoney</ref>
 			<ref type="application">SayNumber</ref>
+			<ref type="application">SayOrdinal</ref>
 			<ref type="application">SayPhonetic</ref>
 			<ref type="function">CHANNEL</ref>
+			<ref type="function">SAYFILES</ref>
+		</see-also>
+	</application>
+	<application name="SayMoney" language="en_US">
+		<synopsis>
+			Say Money.
+		</synopsis>
+		<syntax>
+			<parameter name="dollars" required="true" />
+		</syntax>
+		<description>
+			<para>This application will play the currency sounds for the given floating point number
+			in the current language. Currently only English and US Dollars is supported.
+			If the channel variable <variable>SAY_DTMF_INTERRUPT</variable> is set to 'true'
+			(case insensitive), then this application will react to DTMF in the same way as
+			<literal>Background</literal>.</para>
+		</description>
+		<see-also>
+			<ref type="application">SayAlpha</ref>
+			<ref type="application">SayNumber</ref>
+			<ref type="application">SayOrdinal</ref>
+			<ref type="application">SayPhonetic</ref>
+			<ref type="function">CHANNEL</ref>
+			<ref type="function">SAYFILES</ref>
 		</see-also>
 	</application>
 	<application name="SayNumber" language="en_US">
@@ -527,8 +562,37 @@
 		<see-also>
 			<ref type="application">SayAlpha</ref>
 			<ref type="application">SayDigits</ref>
+			<ref type="application">SayMoney</ref>
 			<ref type="application">SayPhonetic</ref>
 			<ref type="function">CHANNEL</ref>
+			<ref type="function">SAYFILES</ref>
+		</see-also>
+	</application>
+	<application name="SayOrdinal" language="en_US">
+		<synopsis>
+			Say Ordinal Number.
+		</synopsis>
+		<syntax>
+			<parameter name="digits" required="true" />
+			<parameter name="gender" />
+		</syntax>
+		<description>
+			<para>This application will play the ordinal sounds that correspond to the given
+			<replaceable>digits</replaceable> (e.g. 1st, 42nd). Currently only English is supported.</para>
+			<para>Optionally, a <replaceable>gender</replaceable> may be
+			specified. This will use the language that is currently set for the channel. See the CHANNEL()
+			function for more information on setting the language for the channel. If the channel variable
+			<variable>SAY_DTMF_INTERRUPT</variable> is set to 'true' (case insensitive), then this
+			application will react to DTMF in the same way as <literal>Background</literal>.</para>
+		</description>
+		<see-also>
+			<ref type="application">SayAlpha</ref>
+			<ref type="application">SayDigits</ref>
+			<ref type="application">SayMoney</ref>
+			<ref type="application">SayNumber</ref>
+			<ref type="application">SayPhonetic</ref>
+			<ref type="function">CHANNEL</ref>
+			<ref type="function">SAYFILES</ref>
 		</see-also>
 	</application>
 	<application name="SayPhonetic" language="en_US">
@@ -547,7 +611,10 @@
 		<see-also>
 			<ref type="application">SayAlpha</ref>
 			<ref type="application">SayDigits</ref>
+			<ref type="application">SayMoney</ref>
 			<ref type="application">SayNumber</ref>
+			<ref type="application">SayOrdinal</ref>
+			<ref type="function">SAYFILES</ref>
 		</see-also>
 	</application>
 	<application name="SetAMAFlags" language="en_US">
@@ -634,6 +701,10 @@
 							be used instead if set</emphasis></para>
 						</argument>
 					</option>
+					<option name="d">
+						<para>Play <literal>dial</literal> indications tone on channel while waiting
+						for digits.</para>
+					</option>
 				</optionlist>
 			</parameter>
 		</syntax>
@@ -929,7 +1000,6 @@ static int pbx_builtin_execiftime(struct ast_channel *chan, const char *data)
 {
 	char *s, *appname;
 	struct ast_timing timing;
-	struct ast_app *app;
 	static const char * const usage = "ExecIfTime requires an argument:\n  <time range>,<days of week>,<days of month>,<months>[,<timezone>]?<appname>[(<appargs>)]";
 
 	if (ast_strlen_zero(data)) {
@@ -967,13 +1037,7 @@ static int pbx_builtin_execiftime(struct ast_channel *chan, const char *data)
 			ast_log(LOG_WARNING, "Failed to find closing parenthesis\n");
 	}
 
-
-	if ((app = pbx_findapp(appname))) {
-		return pbx_exec(chan, app, S_OR(s, ""));
-	} else {
-		ast_log(LOG_WARNING, "Cannot locate application %s\n", appname);
-		return -1;
-	}
+	return ast_pbx_exec_application(chan, appname, S_OR(s, ""));
 }
 
 /*!
@@ -1259,6 +1323,28 @@ static int pbx_builtin_gotoif(struct ast_channel *chan, const char *data)
 	return pbx_builtin_goto(chan, branch);
 }
 
+/*!
+ * \brief Determine if DTMF interruption was requested.
+ *
+ * If the SAY_DTMF_INTERRUPT channel variable is truthy, the caller has
+ * requested DTMF interruption be enabled.
+ *
+ * \param chan the channel to examine
+ *
+ * \retval -1 if DTMF interruption was requested
+ * \retval  0 if DTMF interruption was not requested
+ */
+static int permit_dtmf_interrupt(struct ast_channel *chan)
+{
+	int interrupt;
+
+	ast_channel_lock(chan);
+	interrupt = ast_true(pbx_builtin_getvar_helper(chan, "SAY_DTMF_INTERRUPT"));
+	ast_channel_unlock(chan);
+
+	return interrupt;
+}
+
 static int pbx_builtin_saynumber(struct ast_channel *chan, const char *data)
 {
 	char tmp[256];
@@ -1266,15 +1352,7 @@ static int pbx_builtin_saynumber(struct ast_channel *chan, const char *data)
 	int number_val;
 	char *options;
 	int res;
-	int interrupt = 0;
-	const char *interrupt_string;
-
-	ast_channel_lock(chan);
-	interrupt_string = pbx_builtin_getvar_helper(chan, "SAY_DTMF_INTERRUPT");
-	if (ast_true(interrupt_string)) {
-		interrupt = 1;
-	}
-	ast_channel_unlock(chan);
+	int interrupt = permit_dtmf_interrupt(chan);
 
 	if (ast_strlen_zero(data)) {
 		ast_log(LOG_WARNING, "SayNumber requires an argument (number)\n");
@@ -1283,7 +1361,7 @@ static int pbx_builtin_saynumber(struct ast_channel *chan, const char *data)
 	ast_copy_string(tmp, data, sizeof(tmp));
 	strsep(&number, ",");
 
-	if (sscanf(tmp, "%d", &number_val) != 1) {
+	if (ast_str_to_int(tmp, &number_val)) {
 		ast_log(LOG_WARNING, "argument '%s' to SayNumber could not be parsed as a number.\n", tmp);
 		return 0;
 	}
@@ -1299,7 +1377,46 @@ static int pbx_builtin_saynumber(struct ast_channel *chan, const char *data)
 
 	res = ast_say_number(chan, number_val, interrupt ? AST_DIGIT_ANY : "", ast_channel_language(chan), options);
 
-	if (res < 0) {
+	if (res < 0 && !ast_check_hangup_locked(chan)) {
+		ast_log(LOG_WARNING, "We were unable to say the number %s, is it too large?\n", tmp);
+	}
+
+	return interrupt ? res : 0;
+}
+
+static int pbx_builtin_sayordinal(struct ast_channel *chan, const char *data)
+{
+	char tmp[256];
+	char *number = tmp;
+	int number_val;
+	char *options;
+	int res;
+	int interrupt = permit_dtmf_interrupt(chan);
+
+	if (ast_strlen_zero(data)) {
+		ast_log(LOG_WARNING, "SayOrdinal requires an argument (number)\n");
+		return -1;
+	}
+	ast_copy_string(tmp, data, sizeof(tmp));
+	strsep(&number, ",");
+
+	if (ast_str_to_int(tmp, &number_val)) {
+		ast_log(LOG_WARNING, "argument '%s' to SayOrdinal could not be parsed as a number.\n", tmp);
+		return 0;
+	}
+
+	options = strsep(&number, ",");
+	if (options) {
+		if ( strcasecmp(options, "f") && strcasecmp(options, "m") &&
+			strcasecmp(options, "c") && strcasecmp(options, "n") ) {
+			ast_log(LOG_WARNING, "SayOrdinal gender option is either 'f', 'm', 'c' or 'n'\n");
+			return -1;
+		}
+	}
+
+	res = ast_say_ordinal(chan, number_val, interrupt ? AST_DIGIT_ANY : "", ast_channel_language(chan), options);
+
+	if (res < 0 && !ast_check_hangup_locked(chan)) {
 		ast_log(LOG_WARNING, "We were unable to say the number %s, is it too large?\n", tmp);
 	}
 
@@ -1309,18 +1426,20 @@ static int pbx_builtin_saynumber(struct ast_channel *chan, const char *data)
 static int pbx_builtin_saydigits(struct ast_channel *chan, const char *data)
 {
 	int res = 0;
-	int interrupt = 0;
-	const char *interrupt_string;
-
-	ast_channel_lock(chan);
-	interrupt_string = pbx_builtin_getvar_helper(chan, "SAY_DTMF_INTERRUPT");
-	if (ast_true(interrupt_string)) {
-		interrupt = 1;
-	}
-	ast_channel_unlock(chan);
 
 	if (data) {
-		res = ast_say_digit_str(chan, data, interrupt ? AST_DIGIT_ANY : "", ast_channel_language(chan));
+		res = ast_say_digit_str(chan, data, permit_dtmf_interrupt(chan) ? AST_DIGIT_ANY : "", ast_channel_language(chan));
+	}
+
+	return res;
+}
+
+static int pbx_builtin_saymoney(struct ast_channel *chan, const char *data)
+{
+	int res = 0;
+
+	if (data) {
+		res = ast_say_money_str(chan, data, permit_dtmf_interrupt(chan) ? AST_DIGIT_ANY : "", ast_channel_language(chan));
 	}
 
 	return res;
@@ -1331,21 +1450,12 @@ static int pbx_builtin_saycharacters_case(struct ast_channel *chan, const char *
 	int res = 0;
 	int sensitivity = 0;
 	char *parse;
-	int interrupt = 0;
-	const char *interrupt_string;
 
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(options);
 		AST_APP_ARG(characters);
 	);
 
-	ast_channel_lock(chan);
-	interrupt_string = pbx_builtin_getvar_helper(chan, "SAY_DTMF_INTERRUPT");
-	if (ast_true(interrupt_string)) {
-		interrupt = 1;
-	}
-	ast_channel_unlock(chan);
-
 	if (ast_strlen_zero(data)) {
 		ast_log(LOG_WARNING, "SayAlphaCase requires two arguments (options, characters)\n");
 		return 0;
@@ -1377,7 +1487,7 @@ static int pbx_builtin_saycharacters_case(struct ast_channel *chan, const char *
 		return 0;
 	}
 
-	res = ast_say_character_str(chan, args.characters, interrupt ? AST_DIGIT_ANY : "", ast_channel_language(chan), sensitivity);
+	res = ast_say_character_str(chan, args.characters, permit_dtmf_interrupt(chan) ? AST_DIGIT_ANY : "", ast_channel_language(chan), sensitivity);
 
 	return res;
 }
@@ -1385,18 +1495,9 @@ static int pbx_builtin_saycharacters_case(struct ast_channel *chan, const char *
 static int pbx_builtin_saycharacters(struct ast_channel *chan, const char *data)
 {
 	int res = 0;
-	int interrupt = 0;
-	const char *interrupt_string;
-
-	ast_channel_lock(chan);
-	interrupt_string = pbx_builtin_getvar_helper(chan, "SAY_DTMF_INTERRUPT");
-	if (ast_true(interrupt_string)) {
-		interrupt = 1;
-	}
-	ast_channel_unlock(chan);
 
 	if (data) {
-		res = ast_say_character_str(chan, data, interrupt ? AST_DIGIT_ANY : "", ast_channel_language(chan), AST_SAY_CASE_NONE);
+		res = ast_say_character_str(chan, data, permit_dtmf_interrupt(chan) ? AST_DIGIT_ANY : "", ast_channel_language(chan), AST_SAY_CASE_NONE);
 	}
 
 	return res;
@@ -1405,18 +1506,11 @@ static int pbx_builtin_saycharacters(struct ast_channel *chan, const char *data)
 static int pbx_builtin_sayphonetic(struct ast_channel *chan, const char *data)
 {
 	int res = 0;
-	int interrupt = 0;
-	const char *interrupt_string;
 
-	ast_channel_lock(chan);
-	interrupt_string = pbx_builtin_getvar_helper(chan, "SAY_DTMF_INTERRUPT");
-	if (ast_true(interrupt_string)) {
-		interrupt = 1;
+	if (data) {
+		res = ast_say_phonetic_str(chan, data, permit_dtmf_interrupt(chan) ? AST_DIGIT_ANY : "", ast_channel_language(chan));
 	}
-	ast_channel_unlock(chan);
 
-	if (data)
-		res = ast_say_phonetic_str(chan, data, interrupt ? AST_DIGIT_ANY : "", ast_channel_language(chan));
 	return res;
 }
 
@@ -1483,7 +1577,9 @@ struct pbx_builtin {
 	{ "SayAlpha",       pbx_builtin_saycharacters },
 	{ "SayAlphaCase",   pbx_builtin_saycharacters_case },
 	{ "SayDigits",      pbx_builtin_saydigits },
+	{ "SayMoney",       pbx_builtin_saymoney },
 	{ "SayNumber",      pbx_builtin_saynumber },
+	{ "SayOrdinal",     pbx_builtin_sayordinal },
 	{ "SayPhonetic",    pbx_builtin_sayphonetic },
 	{ "SetAMAFlags",    pbx_builtin_setamaflags },
 	{ "Wait",           pbx_builtin_wait },
