@@ -93,11 +93,22 @@ struct ast_stream_topology {
 	AST_VECTOR(, struct ast_stream *) streams;
 };
 
+const char *ast_stream_state_map[] = {
+       [AST_STREAM_STATE_REMOVED] = "removed",
+       [AST_STREAM_STATE_SENDRECV] = "sendrecv",
+       [AST_STREAM_STATE_SENDONLY] = "sendonly",
+       [AST_STREAM_STATE_RECVONLY] = "recvonly",
+       [AST_STREAM_STATE_INACTIVE] = "inactive",
+};
+
+#define MIN_STREAM_NAME_LEN 16
+
 struct ast_stream *ast_stream_alloc(const char *name, enum ast_media_type type)
 {
 	struct ast_stream *stream;
+	size_t name_len = MAX(strlen(S_OR(name, "")), MIN_STREAM_NAME_LEN); /* Ensure there is enough room for 'removed' or a type-position */
 
-	stream = ast_calloc(1, sizeof(*stream) + strlen(S_OR(name, "")) + 1);
+	stream = ast_calloc(1, sizeof(*stream) + name_len + 1);
 	if (!stream) {
 		return NULL;
 	}
@@ -107,22 +118,28 @@ struct ast_stream *ast_stream_alloc(const char *name, enum ast_media_type type)
 	stream->group = -1;
 	strcpy(stream->name, S_OR(name, "")); /* Safe */
 
+	stream->formats = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!stream->formats) {
+		ast_free(stream);
+		return NULL;
+	}
+
 	return stream;
 }
 
 struct ast_stream *ast_stream_clone(const struct ast_stream *stream, const char *name)
 {
 	struct ast_stream *new_stream;
-	size_t stream_size;
 	const char *stream_name;
+	size_t name_len;
 
 	if (!stream) {
 		return NULL;
 	}
 
 	stream_name = name ?: stream->name;
-	stream_size = sizeof(*stream) + strlen(stream_name) + 1;
-	new_stream = ast_calloc(1, stream_size);
+	name_len = MAX(strlen(S_OR(stream_name, "")), MIN_STREAM_NAME_LEN); /* Ensure there is enough room for 'removed' or a type-position */
+	new_stream = ast_calloc(1, sizeof(*stream) + name_len + 1);
 	if (!new_stream) {
 		return NULL;
 	}
@@ -130,9 +147,13 @@ struct ast_stream *ast_stream_clone(const struct ast_stream *stream, const char
 	memcpy(new_stream, stream, sizeof(*new_stream));
 	strcpy(new_stream->name, stream_name); /* Safe */
 	new_stream->group = -1;
-	if (new_stream->formats) {
-		ao2_ref(new_stream->formats, +1);
+
+	new_stream->formats = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!new_stream->formats) {
+		ast_free(new_stream);
+		return NULL;
 	}
+	ast_format_cap_append_from_cap(new_stream->formats, stream->formats, AST_MEDIA_TYPE_UNKNOWN);
 
 	new_stream->metadata = ast_stream_get_metadata_list(stream);
 
@@ -178,13 +199,41 @@ void ast_stream_set_type(struct ast_stream *stream, enum ast_media_type type)
 	stream->type = type;
 }
 
-struct ast_format_cap *ast_stream_get_formats(const struct ast_stream *stream)
+const struct ast_format_cap *ast_stream_get_formats(const struct ast_stream *stream)
 {
 	ast_assert(stream != NULL);
 
 	return stream->formats;
 }
 
+const char *ast_stream_to_str(const struct ast_stream *stream, struct ast_str **buf)
+{
+	if (!buf || !*buf) {
+		return "";
+	}
+
+	if (!stream) {
+		ast_str_append(buf, 0, "(null stream)");
+		return ast_str_buffer(*buf);
+	}
+
+	ast_str_append(buf, 0, "%d:%s:%s:%s ",
+		stream->position,
+		S_OR(stream->name, "noname"),
+		ast_codec_media_type2str(stream->type),
+		ast_stream_state_map[stream->state]);
+	ast_format_cap_append_names(stream->formats, buf);
+
+	return ast_str_buffer(*buf);
+}
+
+int ast_stream_get_format_count(const struct ast_stream *stream)
+{
+       ast_assert(stream != NULL);
+
+       return stream->formats ? ast_format_cap_count(stream->formats) : 0;
+}
+
 void ast_stream_set_formats(struct ast_stream *stream, struct ast_format_cap *caps)
 {
 	ast_assert(stream != NULL);
@@ -205,6 +254,7 @@ void ast_stream_set_state(struct ast_stream *stream, enum ast_stream_state state
 	ast_assert(stream != NULL);
 
 	stream->state = state;
+
 }
 
 const char *ast_stream_state2str(enum ast_stream_state state)
@@ -461,6 +511,10 @@ int ast_stream_topology_append_stream(struct ast_stream_topology *topology, stru
 
 	stream->position = AST_VECTOR_SIZE(&topology->streams) - 1;
 
+	if (ast_strlen_zero(stream->name)) {
+		snprintf(stream->name, MIN_STREAM_NAME_LEN, "%s-%d", ast_codec_media_type2str(stream->type), stream->position);
+	}
+
 	return AST_VECTOR_SIZE(&topology->streams) - 1;
 }
 
@@ -471,6 +525,22 @@ int ast_stream_topology_get_count(const struct ast_stream_topology *topology)
 	return AST_VECTOR_SIZE(&topology->streams);
 }
 
+int ast_stream_topology_get_active_count(const struct ast_stream_topology *topology)
+{
+       int i;
+       int count = 0;
+       ast_assert(topology != NULL);
+
+       for (i = 0; i < AST_VECTOR_SIZE(&topology->streams); i++) {
+               struct ast_stream *stream = AST_VECTOR_GET(&topology->streams, i);
+               if (stream->state != AST_STREAM_STATE_REMOVED) {
+                       count++;
+               }
+       }
+
+       return count;
+}
+
 struct ast_stream *ast_stream_topology_get_stream(
 	const struct ast_stream_topology *topology, unsigned int stream_num)
 {
@@ -501,6 +571,10 @@ int ast_stream_topology_set_stream(struct ast_stream_topology *topology,
 		return AST_VECTOR_APPEND(&topology->streams, stream);
 	}
 
+	if (ast_strlen_zero(stream->name)) {
+		snprintf(stream->name, MIN_STREAM_NAME_LEN, "%s-%d", ast_codec_media_type2str(stream->type), stream->position);
+	}
+
 	return AST_VECTOR_REPLACE(&topology->streams, position, stream);
 }
 
@@ -559,20 +633,23 @@ struct ast_stream_topology *ast_stream_topology_create_from_format_cap(
 			return NULL;
 		}
 
-		stream = ast_stream_alloc(ast_codec_media_type2str(type), type);
+		stream = ast_stream_alloc(NULL, type);
 		if (!stream) {
 			ao2_cleanup(new_cap);
 			ast_stream_topology_free(topology);
 			return NULL;
 		}
-		/* We're transferring the initial ref so no bump needed */
-		stream->formats = new_cap;
+
+		ast_stream_set_formats(stream, new_cap);
+		ao2_ref(new_cap, -1);
 		stream->state = AST_STREAM_STATE_SENDRECV;
 		if (ast_stream_topology_append_stream(topology, stream) == -1) {
 			ast_stream_free(stream);
 			ast_stream_topology_free(topology);
 			return NULL;
 		}
+
+		snprintf(stream->name, MIN_STREAM_NAME_LEN, "%s-%d", ast_codec_media_type2str(stream->type), stream->position);
 	}
 
 	return topology;
@@ -606,6 +683,32 @@ struct ast_format_cap *ast_format_cap_from_stream_topology(
 	return caps;
 }
 
+const char *ast_stream_topology_to_str(const struct ast_stream_topology *topology,
+       struct ast_str **buf)
+{
+       int i;
+
+       if (!buf ||!*buf) {
+               return "";
+       }
+
+       if (!topology) {
+               ast_str_append(buf, 0, "(null topology)");
+               return ast_str_buffer(*buf);
+       }
+
+       for (i = 0; i < AST_VECTOR_SIZE(&topology->streams); i++) {
+               struct ast_stream *stream;
+
+               stream = AST_VECTOR_GET(&topology->streams, i);
+               ast_str_append(buf, 0, " <");
+               ast_stream_to_str(stream, buf);
+               ast_str_append(buf, 0, ">");
+       }
+
+       return ast_str_buffer(*buf);
+}
+
 struct ast_stream *ast_stream_topology_get_first_stream_by_type(
 	const struct ast_stream_topology *topology,
 	enum ast_media_type type)
