@@ -83,6 +83,8 @@
 /*** MODULEINFO
 	<depend>sqlite</depend>
 	<support_level>deprecated</support_level>
+	<deprecated_in>16</deprecated_in>
+	<removed_in>19</removed_in>
  ***/
 
 #include "asterisk.h"
