@@ -35,6 +35,8 @@
 
 #include <dirent.h>                 /* for closedir, opendir, readdir, DIR */
 
+#define OPENSSL_SUPPRESS_DEPRECATED 1
+
 #include <openssl/aes.h>            /* for AES_decrypt, AES_encrypt, AES_set... */
 #include <openssl/err.h>            /* for ERR_print_errors_fp */
 #include <openssl/ssl.h>            /* for NID_sha1, RSA */
@@ -161,7 +163,7 @@ struct ast_key * AST_OPTIONAL_API_NAME(ast_key_get)(const char *kname, int ktype
  * \param ifd incoming file descriptor
  * \param ofd outgoing file descriptor
  * \param not2
- * \retval key on success.
+ * \return key on success.
  * \retval NULL on failure.
 */
 static struct ast_key *try_load_key(const char *dir, const char *fname, int ifd, int ofd, int *not2)
@@ -487,7 +489,6 @@ void AST_OPTIONAL_API_NAME(ast_aes_decrypt)(const unsigned char *in, unsigned ch
  * \brief refresh RSA keys from file
  * \param ifd file descriptor
  * \param ofd file descriptor
- * \return void
 */
 static void crypto_load(int ifd, int ofd)
 {
@@ -546,7 +547,7 @@ static void md52sum(char *sum, unsigned char *md5)
  * \param e CLI command
  * \param cmd
  * \param a list of CLI arguments
- * \return CLI_SUCCESS
+ * \retval CLI_SUCCESS
 */
 static char *handle_cli_keys_show(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
@@ -592,7 +593,7 @@ static char *handle_cli_keys_show(struct ast_cli_entry *e, int cmd, struct ast_c
  * \param e CLI command
  * \param cmd
  * \param a list of CLI arguments
- * \return CLI_SUCCESS
+ * \retval CLI_SUCCESS
 */
 static char *handle_cli_keys_init(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
