@@ -505,6 +505,7 @@ static void delete_routes(void)
 
 /*! \brief Build a route structure and add it to the list of available http routes
 	\param pp_file File to link to the route
+	\param profile
 	\param user User to link to the route (NULL means static route)
 	\param uri URI of the route
 */
@@ -607,13 +608,14 @@ static void build_profile(const char *name, struct ast_variable *v)
 		if (!strcasecmp(v->name, "mime_type")) {
 			ast_string_field_set(profile, default_mime_type, v->value);
 		} else if (!strcasecmp(v->name, "setvar")) {
-			char *value_copy = ast_strdupa(v->value);
+			char value_copy[strlen(v->value) + 1];
 
 			AST_DECLARE_APP_ARGS(args,
 				AST_APP_ARG(varname);
 				AST_APP_ARG(varval);
 			);
 
+			strcpy(value_copy, v->value); /* safe */
 			AST_NONSTANDARD_APP_ARGS(args, value_copy, '=');
 			do {
 				if (ast_strlen_zero(args.varname) || ast_strlen_zero(args.varval))
@@ -629,7 +631,7 @@ static void build_profile(const char *name, struct ast_variable *v)
 		} else {
 			struct phoneprov_file *pp_file;
 			char *file_extension;
-			char *value_copy = ast_strdupa(v->value);
+			char value_copy[strlen(v->value) + 1];
 
 			AST_DECLARE_APP_ARGS(args,
 				AST_APP_ARG(filename);
@@ -644,6 +646,7 @@ static void build_profile(const char *name, struct ast_variable *v)
 			if ((file_extension = strrchr(pp_file->format, '.')))
 				file_extension++;
 
+			strcpy(value_copy, v->value); /* safe */
 			AST_STANDARD_APP_ARGS(args, value_copy);
 
 			/* Mime type order of preference
@@ -1072,7 +1075,7 @@ static int pp_each_extension_helper(struct ast_channel *chan, const char *cmd, c
 	AST_STANDARD_APP_ARGS(args, data);
 
 	if (ast_strlen_zero(args.mac) || ast_strlen_zero(args.template)) {
-		ast_log(LOG_WARNING, "PP_EACH_EXTENSION requries both a macaddress and template filename.\n");
+		ast_log(LOG_WARNING, "PP_EACH_EXTENSION requires both a macaddress and template filename.\n");
 		return 0;
 	}
 
