@@ -4,6 +4,7 @@
  * Copyright (C) 2013, Fairview 5 Engineering, LLC
  *
  * George Joseph <george.joseph@fairview5.com>
+ * José Lopes <jose.lopes@nfon.com>
  *
  * See http://www.asterisk.org for more information about
  * the Asterisk project. Please do not directly contact
@@ -46,7 +47,10 @@
 			<parameter name="action" required="true">
 				<enumlist>
 					<enum name="read"><para>Returns instance <replaceable>number</replaceable>
-					of header <replaceable>name</replaceable>.</para></enum>
+					of header <replaceable>name</replaceable>. A <literal>*</literal>
+					may be appended to <replaceable>name</replaceable> to iterate over all 
+					headers <emphasis>beginning with</emphasis> <replaceable>name</replaceable>.
+					</para></enum>
 
 					<enum name="add"><para>Adds a new header <replaceable>name</replaceable>
 					to this session.</para></enum>
@@ -81,64 +85,172 @@
 			channel. One exception is that you can read headers that you have already
 			added on the outbound channel.</para>
 			<para>Examples:</para>
-			<para>;</para>
-			<para>; Set 'somevar' to the value of the 'From' header.</para>
-			<para>exten => 1,1,Set(somevar=${PJSIP_HEADER(read,From)})</para>
-			<para>;</para>
-			<para>; Set 'via2' to the value of the 2nd 'Via' header.</para>
-			<para>exten => 1,1,Set(via2=${PJSIP_HEADER(read,Via,2)})</para>
-			<para>;</para>
-			<para>; Add an 'X-Myheader' header with the value of 'myvalue'.</para>
-			<para>exten => 1,1,Set(PJSIP_HEADER(add,X-MyHeader)=myvalue)</para>
-			<para>;</para>
-			<para>; Add an 'X-Myheader' header with an empty value.</para>
-			<para>exten => 1,1,Set(PJSIP_HEADER(add,X-MyHeader)=)</para>
-			<para>;</para>
-			<para>; Update the value of the header named 'X-Myheader' to 'newvalue'.</para>
-			<para>; 'X-Myheader' must already exist or the call will fail.</para>
-			<para>exten => 1,1,Set(PJSIP_HEADER(update,X-MyHeader)=newvalue)</para>
-			<para>;</para>
-			<para>; Remove all headers whose names exactly match 'X-MyHeader'.</para>
-			<para>exten => 1,1,Set(PJSIP_HEADER(remove,X-MyHeader)=)</para>
-			<para>;</para>
-			<para>; Remove all headers that begin with 'X-My'.</para>
-			<para>exten => 1,1,Set(PJSIP_HEADER(remove,X-My*)=)</para>
-			<para>;</para>
-			<para>; Remove all previously added headers.</para>
-			<para>exten => 1,1,Set(PJSIP_HEADER(remove,*)=)</para>
-			<para>;</para>
-
+			<example title="Set somevar to the value of the From header">
+			exten => 1,1,Set(somevar=${PJSIP_HEADER(read,From)})
+			</example>
+			<example title="Set via2 to the value of the 2nd Via header">
+			exten => 1,1,Set(via2=${PJSIP_HEADER(read,Via,2)})
+			</example>
+			<example title="Set xhdr to the value of the 1st X-header">
+			exten => 1,1,Set(xhdr=${PJSIP_HEADER(read,X-*,1)})
+			</example>
+			<example title="Add an X-Myheader header with the value of myvalue">
+			exten => 1,1,Set(PJSIP_HEADER(add,X-MyHeader)=myvalue)
+			</example>
+			<example title="Add an X-Myheader header with an empty value">
+			exten => 1,1,Set(PJSIP_HEADER(add,X-MyHeader)=)
+			</example>
+			<example title="Update the value of the header named X-Myheader to newvalue">
+			; 'X-Myheader' must already exist or the call will fail.
+			exten => 1,1,Set(PJSIP_HEADER(update,X-MyHeader)=newvalue)
+			</example>
+			<example title="Remove all headers whose names exactly match X-MyHeader">
+			exten => 1,1,Set(PJSIP_HEADER(remove,X-MyHeader)=)
+			</example>
+			<example title="Remove all headers that begin with X-My">
+			exten => 1,1,Set(PJSIP_HEADER(remove,X-My*)=)
+			</example>
+			<example title="Remove all previously added headers">
+			exten => 1,1,Set(PJSIP_HEADER(remove,*)=)
+			</example>
 			<note><para>The <literal>remove</literal> action can be called by reading
-			<emphasis>or</emphasis> writing PJSIP_HEADER.</para>
-			<para>;</para>
-			<para>; Display the number of headers removed</para>
-			<para>exten => 1,1,Verbose( Removed ${PJSIP_HEADER(remove,X-MyHeader)} headers)</para>
-			<para>;</para>
-			<para>; Set a variable to the number of headers removed</para>
-			<para>exten => 1,1,Set(count=${PJSIP_HEADER(remove,X-MyHeader)})</para>
-			<para>;</para>
-			<para>; Just remove them ignoring any count</para>
-			<para>exten => 1,1,Set(=${PJSIP_HEADER(remove,X-MyHeader)})</para>
-			<para>exten => 1,1,Set(PJSIP_HEADER(remove,X-MyHeader)=)</para>
-			<para>;</para>
-			</note>
+			<emphasis>or</emphasis> writing PJSIP_HEADER.</para></note>
+			<example title="Display the number of headers removed">
+			exten => 1,1,Verbose( Removed ${PJSIP_HEADER(remove,X-MyHeader)} headers)
+			</example>
+			<example title="Set a variable to the number of headers removed">
+			exten => 1,1,Set(count=${PJSIP_HEADER(remove,X-MyHeader)})
+			</example>
+			<example title="Just remove them ignoring any count">
+			exten => 1,1,Set(=${PJSIP_HEADER(remove,X-MyHeader)})
+			exten => 1,1,Set(PJSIP_HEADER(remove,X-MyHeader)=)
+			</example>
 
 			<note><para>If you call PJSIP_HEADER in a normal dialplan context you'll be
 			operating on the <emphasis>caller's (incoming)</emphasis> channel which
 			may not be what you want. To operate on the <emphasis>callee's (outgoing)</emphasis>
-			channel call PJSIP_HEADER in a pre-dial handler. </para>
-			<para>Example:</para>
-			<para>;</para>
-			<para>[handler]</para>
-			<para>exten => addheader,1,Set(PJSIP_HEADER(add,X-MyHeader)=myvalue)</para>
-			<para>exten => addheader,2,Set(PJSIP_HEADER(add,X-MyHeader2)=myvalue2)</para>
-			<para>;</para>
-			<para>[somecontext]</para>
-			<para>exten => 1,1,Dial(PJSIP/${EXTEN},,b(handler^addheader^1))</para>
-			<para>;</para>
-			</note>
+			channel call PJSIP_HEADER in a pre-dial handler. </para></note>
+			<example title="Set headers on callee channel">
+			[handler]
+			exten => addheader,1,Set(PJSIP_HEADER(add,X-MyHeader)=myvalue)
+			exten => addheader,2,Set(PJSIP_HEADER(add,X-MyHeader2)=myvalue2)
+
+			[somecontext]
+			exten => 1,1,Dial(PJSIP/${EXTEN},,b(handler^addheader^1))
+			</example>
 		</description>
 	</function>
+	<function name="PJSIP_HEADERS" language="en_US">
+		<since>
+			<version>16.20.0</version>
+			<version>18.6.0</version>
+			<version>19.0.0</version>
+		</since>
+		<synopsis>
+			Gets the list of SIP header names from an INVITE message.
+		</synopsis>
+		<syntax>
+			<parameter name="prefix">
+				<para>If specified, only the headers matching the given prefix are returned.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Returns a comma-separated list of header names (without values) from the
+			INVITE message. Multiple headers with the same name are included in the list only once.
+			</para>
+			<para>For example, <literal>${PJSIP_HEADERS(Co)}</literal> might return
+			<literal>Contact,Content-Length,Content-Type</literal>. As a practical example,
+			you may use <literal>${PJSIP_HEADERS(X-)}</literal> to enumerate optional extended
+			headers.</para>
+		</description>
+		<see-also>
+			<ref type="function">PJSIP_HEADER</ref>
+		</see-also>
+	</function>
+	<function name="PJSIP_RESPONSE_HEADER" language="en_US">
+		<synopsis>
+			Gets headers of 200 response from an outbound PJSIP channel.
+		</synopsis>
+		<syntax>
+			<parameter name="action" required="true">
+				<enumlist>
+					<enum name="read">
+						<para>Returns instance <replaceable>number</replaceable>
+						of response header <replaceable>name</replaceable>.</para>
+					</enum>
+				</enumlist>
+			</parameter>
+
+			<parameter name="name" required="true">
+				<para>The <replaceable>name</replaceable> of the response header.
+				A <literal>*</literal> can be appended to the <replaceable>name</replaceable>
+				to iterate over all response headers <emphasis>beginning with</emphasis>
+				<replaceable>name</replaceable>.</para>
+			</parameter>
+
+			<parameter name="number" required="false">
+				<para>If there's more than 1 header with the same name, this specifies which header
+				to read.  If not specified, defaults to <literal>1</literal> meaning
+				the first matching header.
+				</para>
+			</parameter>
+
+		</syntax>
+		<description>
+			<para>PJSIP_RESPONSE_HEADER allows you to read specific SIP headers of 200 response
+			from the outbound PJSIP channel.</para>
+			<para>Examples:</para>
+			<example title="Set 'somevar' to the value of the 'From' header">
+				exten => 1,1,Set(somevar=${PJSIP_RESPONSE_HEADER(read,From)})
+			</example>
+			<example title="Set 'via2' to the value of the 2nd 'Via' header">
+				exten => 1,1,Set(via2=${PJSIP_RESPONSE_HEADER(read,Via,2)})
+			</example>
+			<example title="Set 'xhdr' to the value of the 1sx X-header">
+				exten => 1,1,Set(xhdr=${PJSIP_RESPONSE_HEADER(read,X-*,1)})
+			</example>
+
+			<note><para>If you call PJSIP_RESPONSE_HEADER in a normal dialplan context you'll be
+			operating on the <emphasis>caller's (incoming)</emphasis> channel which
+			may not be what you want. To operate on the <emphasis>callee's (outgoing)</emphasis>
+			channel call PJSIP_RESPONSE_HEADER in a pre-connect handler.</para>
+			</note>
+			<example title="Usage on pre-connect handler">
+				[handler]
+				exten => readheader,1,NoOp(PJSIP_RESPONSE_HEADER(read,X-MyHeader))
+				[somecontext]
+				exten => 1,1,Dial(PJSIP/${EXTEN},,U(handler^readheader^1))
+			</example>
+		</description>
+		<see-also>
+			<ref type="function">PJSIP_RESPONSE_HEADERS</ref>
+			<ref type="function">PJSIP_HEADER</ref>
+		</see-also>
+	</function>
+	<function name="PJSIP_RESPONSE_HEADERS" language="en_US">
+		<synopsis>
+			Gets the list of SIP header names from the 200 response of INVITE message.
+		</synopsis>
+		<syntax>
+			<parameter name="prefix">
+				<para>If specified, only the headers matching the given prefix are returned.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Returns a comma-separated list of header names (without values) from the 200
+			response of INVITE message. Multiple headers with the same name are included in the
+			list only once.</para>
+			<para>For example, <literal>${PJSIP_RESPONSE_HEADERS(Co)}</literal> might return
+			<literal>Contact,Content-Length,Content-Type</literal>. As a practical example,
+			you may use <literal>${PJSIP_RESPONSE_HEADERS(X-)}</literal> to enumerate optional
+			extended headers.</para>
+		</description>
+		<see-also>
+			<ref type="function">PJSIP_RESPONSE_HEADER</ref>
+			<ref type="function">PJSIP_HEADERS</ref>
+		</see-also>
+	</function>
+
  ***/
 
 /*! \brief Linked list for accumulating headers */
@@ -152,6 +264,10 @@ AST_LIST_HEAD_NOLOCK(hdr_list, hdr_list_entry);
 static const struct ast_datastore_info header_datastore = {
 	.type = "header_datastore",
 };
+/*! \brief Datastore for saving response headers */
+static const struct ast_datastore_info response_header_datastore = {
+	.type = "response_header_datastore",
+};
 
 /*! \brief Data structure used for ast_sip_push_task_wait_serializer  */
 struct header_data {
@@ -161,6 +277,7 @@ struct header_data {
 	char *buf;
 	int header_number;
 	size_t len;
+	const struct ast_datastore_info *header_datastore;
 };
 
 /*!
@@ -215,6 +332,42 @@ static int incoming_request(struct ast_sip_session *session, pjsip_rx_data * rda
 	return 0;
 }
 
+/*!
+ * \internal
+ * \brief Session supplement callback on an incoming INVITE response
+ *
+ * Retrieve the response_header_datastore from the session or create one if it doesn't exist.
+ * Create and initialize the list if needed.
+ * Insert the headers.
+ */
+static void incoming_response(struct ast_sip_session *session, pjsip_rx_data * rdata)
+{
+	pj_pool_t *pool = session->inv_session->dlg->pool;
+	RAII_VAR(struct ast_datastore *, datastore,
+			 ast_sip_session_get_datastore(session, response_header_datastore.type), ao2_cleanup);
+	pjsip_status_line status = rdata->msg_info.msg->line.status;
+
+	/* Skip responses different of 200 OK, when 2xx is received. */
+	if (session->inv_session->state != PJSIP_INV_STATE_CONNECTING || status.code!=200) {
+		return;
+	}
+
+	if (!datastore) {
+		if (!(datastore =
+			  ast_sip_session_alloc_datastore(&response_header_datastore, response_header_datastore.type))
+			||
+			!(datastore->data = pj_pool_alloc(pool, sizeof(struct hdr_list))) ||
+			ast_sip_session_add_datastore(session, datastore)) {
+			ast_log(AST_LOG_ERROR, "Unable to create datastore for header functions.\n");
+			return;
+		}
+		AST_LIST_HEAD_INIT_NOLOCK((struct hdr_list *) datastore->data);
+	}
+	insert_headers(pool, (struct hdr_list *) datastore->data, rdata->msg_info.msg);
+
+	return;
+}
+
 /*!
  * \internal
  * \brief Search list for nth occurrence of specific header.
@@ -240,10 +393,95 @@ static pjsip_hdr *find_header(struct hdr_list *list, const char *header_name,
 	return hdr;
 }
 
+/*!
+ * \internal
+ * \brief Implements PJSIP_HEADERS/PJSIP_RESPONSE_HEADERS by searching for the requested header prefix.
+ *
+ * Retrieve the header_datastore.
+ * Search for the all matching headers.
+ * Validate the pjsip_hdr found.
+ * Parse pjsip_hdr into a name and copy to the buffer.
+ * Return the value.
+ */
+static int read_headers(void *obj)
+{
+	struct header_data *data = obj;
+	size_t len = strlen(data->header_name);
+	pjsip_hdr *hdr = NULL;
+	char *pj_hdr_string;
+	int pj_hdr_string_len;
+	char *p;
+	char *pos;
+	size_t plen, wlen = 0;
+	struct hdr_list_entry *le;
+	struct hdr_list *list;
+
+	RAII_VAR(struct ast_datastore *, datastore,
+			 ast_sip_session_get_datastore(data->channel->session, data->header_datastore->type),
+			 ao2_cleanup);
+
+	if (!datastore || !datastore->data) {
+		ast_debug(1, "There was no datastore from which to read headers.\n");
+		return -1;
+	}
+
+	list = datastore->data;
+	pj_hdr_string = ast_alloca(data->len);
+	AST_LIST_TRAVERSE(list, le, nextptr) {
+		if (pj_strnicmp2(&le->hdr->name, data->header_name, len) == 0) {
+			/* Found matched header, append to buf */
+			hdr = le->hdr;
+
+			pj_hdr_string_len = pjsip_hdr_print_on(hdr, pj_hdr_string, data->len - 1);
+			if (pj_hdr_string_len == -1) {
+				ast_log(AST_LOG_ERROR,
+					"Not enough buffer space in pjsip_hdr_print_on\n");
+				return -1;
+			}
+			pj_hdr_string[pj_hdr_string_len] = '\0';
+			p = strchr(pj_hdr_string, ':');
+			if (!p) {
+				ast_log(AST_LOG_WARNING,
+					"A malformed header was returned from pjsip_hdr_print_on\n");
+				continue;
+			}
+
+			pj_hdr_string[p - pj_hdr_string] = '\0';
+			p = ast_strip(pj_hdr_string);
+			plen = strlen(p);
+			if (wlen + plen + 1 > data->len) {
+				ast_log(AST_LOG_ERROR,
+						"Buffer isn't big enough to hold header value.  %zu > %zu\n", plen + 1,
+						data->len);
+				return -1;
+			}
+			pos = strstr(data->buf, p);
+			if (pos && pos[1] == ',') {
+				if (pos == data->buf) {
+					continue;
+				} else if (pos[-1] == ',') {
+					continue;
+				}
+			}
+			ast_copy_string(data->buf + wlen, p, data->len - wlen);
+			wlen += plen;
+			ast_copy_string(data->buf + wlen, ",", data->len - wlen);
+			wlen++;
+		}
+	}
+
+	if (wlen == 0) {
+		ast_debug(1, "There was no header named %s.\n", data->header_name);
+		return -1;
+	} else {
+		data->buf[wlen-1] = '\0';
+	}
+	return 0;
+}
 
 /*!
  * \internal
- * \brief Implements PJSIP_HEADER 'read' by searching the for the requested header.
+ * \brief Implements PJSIP_HEADER/PJSIP_RESPONSE_HEADER 'read' by searching the for the requested header.
  *
  * Retrieve the header_datastore.
  * Search for the nth matching header.
@@ -254,13 +492,17 @@ static pjsip_hdr *find_header(struct hdr_list *list, const char *header_name,
 static int read_header(void *obj)
 {
 	struct header_data *data = obj;
+	size_t len = strlen(data->header_name);
 	pjsip_hdr *hdr = NULL;
 	char *pj_hdr_string;
-	size_t pj_hdr_string_len;
+	int pj_hdr_string_len;
 	char *p;
 	size_t plen;
+	struct hdr_list_entry *le;
+	struct hdr_list *list;
+	int i = 1;
 	RAII_VAR(struct ast_datastore *, datastore,
-			 ast_sip_session_get_datastore(data->channel->session, header_datastore.type),
+			 ast_sip_session_get_datastore(data->channel->session, data->header_datastore->type),
 			 ao2_cleanup);
 
 	if (!datastore || !datastore->data) {
@@ -268,8 +510,20 @@ static int read_header(void *obj)
 		return -1;
 	}
 
-	hdr = find_header((struct hdr_list *) datastore->data, data->header_name,
-					  data->header_number);
+	list = datastore->data;
+	AST_LIST_TRAVERSE(list, le, nextptr) {
+		if (data->header_name[len - 1] == '*') {
+			if (pj_strnicmp2(&le->hdr->name, data->header_name, len - 1) == 0 && i++ == data->header_number) {
+				hdr = le->hdr;
+				break;
+			}
+		} else {
+			if (pj_stricmp2(&le->hdr->name, data->header_name) == 0 && i++ == data->header_number) {
+				hdr = le->hdr;
+				break;
+			}
+		}
+	}
 
 	if (!hdr) {
 		ast_debug(1, "There was no header named %s.\n", data->header_name);
@@ -277,7 +531,13 @@ static int read_header(void *obj)
 	}
 
 	pj_hdr_string = ast_alloca(data->len);
-	pj_hdr_string_len = pjsip_hdr_print_on(hdr, pj_hdr_string, data->len);
+	pj_hdr_string_len = pjsip_hdr_print_on(hdr, pj_hdr_string, data->len - 1);
+	if (pj_hdr_string_len == -1) {
+		ast_log(AST_LOG_ERROR,
+			"Not enough buffer space in pjsip_hdr_print_on\n");
+		return -1;
+	}
+
 	pj_hdr_string[pj_hdr_string_len] = '\0';
 
 	p = strchr(pj_hdr_string, ':');
@@ -304,7 +564,7 @@ static int read_header(void *obj)
 
 /*!
  * \internal
- * \brief Implements PJSIP_HEADER 'add' by inserting the specified header into thge list.
+ * \brief Implements PJSIP_HEADER 'add' by inserting the specified header into the list.
  *
  * Retrieve the header_datastore from the session or create one if it doesn't exist.
  * Create and initialize the list if needed.
@@ -323,11 +583,11 @@ static int add_header(void *obj)
 	struct hdr_list *list;
 
 	RAII_VAR(struct ast_datastore *, datastore,
-			 ast_sip_session_get_datastore(session, header_datastore.type), ao2_cleanup);
+			 ast_sip_session_get_datastore(session, data->header_datastore->type), ao2_cleanup);
 
 	if (!datastore) {
-		if (!(datastore = ast_sip_session_alloc_datastore(&header_datastore,
-														  header_datastore.type))
+		if (!(datastore = ast_sip_session_alloc_datastore(data->header_datastore,
+														data->header_datastore->type))
 			|| !(datastore->data = pj_pool_alloc(pool, sizeof(struct hdr_list)))
 			|| ast_sip_session_add_datastore(session, datastore)) {
 			ast_log(AST_LOG_ERROR, "Unable to create datastore for header functions.\n");
@@ -366,7 +626,7 @@ static int update_header(void *obj)
 	struct header_data *data = obj;
 	pjsip_hdr *hdr = NULL;
 	RAII_VAR(struct ast_datastore *, datastore,
-			 ast_sip_session_get_datastore(data->channel->session, header_datastore.type),
+			 ast_sip_session_get_datastore(data->channel->session, data->header_datastore->type),
 			 ao2_cleanup);
 
 	if (!datastore || !datastore->data) {
@@ -403,7 +663,7 @@ static int remove_header(void *obj)
 	struct hdr_list_entry *le;
 	int removed_count = 0;
 	RAII_VAR(struct ast_datastore *, datastore,
-			 ast_sip_session_get_datastore(data->channel->session, header_datastore.type),
+			 ast_sip_session_get_datastore(data->channel->session, data->header_datastore->type),
 			 ao2_cleanup);
 
 	if (!datastore || !datastore->data) {
@@ -435,12 +695,77 @@ static int remove_header(void *obj)
 }
 
 /*!
- * \brief Implements function 'read' callback.
+ * \brief Read list of unique SIP headers
+ */
+static int func_read_headers(struct ast_channel *chan, const char *function, char *data, char *buf, size_t len)
+{
+	struct ast_sip_channel_pvt *channel = chan ? ast_channel_tech_pvt(chan) : NULL;
+	struct header_data header_data;
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(header_pattern);
+	);
+	AST_STANDARD_APP_ARGS(args, data);
+
+	if (!chan || strncmp(ast_channel_name(chan), "PJSIP/", 6)) {
+		ast_log(LOG_ERROR, "This function requires a PJSIP channel.\n");
+		return -1;
+	}
+
+	if (ast_strlen_zero(args.header_pattern)) {
+		ast_log(AST_LOG_ERROR, "This function requires a pattern.\n");
+		return -1;
+	}
+
+	header_data.channel = channel;
+	header_data.header_name = args.header_pattern;
+	header_data.header_value = NULL;
+	header_data.buf = buf;
+	header_data.len = len;
+	header_data.header_datastore = &header_datastore;
+
+	return ast_sip_push_task_wait_serializer(channel->session->serializer, read_headers, &header_data);
+
+}
+
+/*!
+ * \brief Read list of unique SIP response headers
+ */
+static int func_response_read_headers(struct ast_channel *chan, const char *function, char *data, char *buf, size_t len)
+{
+	struct ast_sip_channel_pvt *channel = chan ? ast_channel_tech_pvt(chan) : NULL;
+	struct header_data header_data;
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(header_pattern);
+	);
+	AST_STANDARD_APP_ARGS(args, data);
+
+	if (!chan || strncmp(ast_channel_name(chan), "PJSIP/", 6)) {
+		ast_log(LOG_ERROR, "This function requires a PJSIP channel.\n");
+		return -1;
+	}
+
+	if (ast_strlen_zero(args.header_pattern)) {
+		ast_log(AST_LOG_ERROR, "This function requires a pattern.\n");
+		return -1;
+	}
+
+	header_data.channel = channel;
+	header_data.header_name = args.header_pattern;
+	header_data.header_value = NULL;
+	header_data.buf = buf;
+	header_data.len = len;
+	header_data.header_datastore = &response_header_datastore;
+
+	return ast_sip_push_task_wait_serializer(channel->session->serializer, read_headers, &header_data);
+
+}
+
+/*!
+ * \brief Implements PJSIP_HEADER function 'read' callback.
  *
  * Valid actions are 'read' and 'remove'.
  */
-static int func_read_header(struct ast_channel *chan, const char *function, char *data,
-							char *buf, size_t len)
+static int func_read_header(struct ast_channel *chan, const char *function, char *data, char *buf, size_t len)
 {
 	struct ast_sip_channel_pvt *channel = chan ? ast_channel_tech_pvt(chan) : NULL;
 	struct header_data header_data;
@@ -478,10 +803,10 @@ static int func_read_header(struct ast_channel *chan, const char *function, char
 	header_data.header_value = NULL;
 	header_data.buf = buf;
 	header_data.len = len;
+	header_data.header_datastore = &header_datastore;
 
 	if (!strcasecmp(args.action, "read")) {
-		return ast_sip_push_task_wait_serializer(channel->session->serializer,
-			read_header, &header_data);
+		return ast_sip_push_task_wait_serializer(channel->session->serializer, read_header, &header_data);
 	} else if (!strcasecmp(args.action, "remove")) {
 		return ast_sip_push_task_wait_serializer(channel->session->serializer,
 			remove_header, &header_data);
@@ -494,7 +819,62 @@ static int func_read_header(struct ast_channel *chan, const char *function, char
 }
 
 /*!
- * \brief Implements function 'write' callback.
+ * \brief Implements PJSIP_RESPONSE_HEADER function 'read' callback.
+ *
+ * Valid actions are 'read'
+ */
+static int func_response_read_header(struct ast_channel *chan, const char *function, char *data, char *buf, size_t len)
+{
+	struct ast_sip_channel_pvt *channel = chan ? ast_channel_tech_pvt(chan) : NULL;
+	struct header_data header_data;
+	int number;
+	AST_DECLARE_APP_ARGS(args,
+						 AST_APP_ARG(action);
+						 AST_APP_ARG(header_name); AST_APP_ARG(header_number););
+	AST_STANDARD_APP_ARGS(args, data);
+
+	if (!channel || strncmp(ast_channel_name(chan), "PJSIP/", 6)) {
+		ast_log(LOG_ERROR, "This function requires a PJSIP channel.\n");
+		return -1;
+	}
+
+	if (ast_strlen_zero(args.action)) {
+		ast_log(AST_LOG_ERROR, "This function requires an action.\n");
+		return -1;
+	}
+	if (ast_strlen_zero(args.header_name)) {
+		ast_log(AST_LOG_ERROR, "This function requires a header name.\n");
+		return -1;
+	}
+	if (!args.header_number) {
+		number = 1;
+	} else {
+		sscanf(args.header_number, "%30d", &number);
+		if (number < 1) {
+			number = 1;
+		}
+	}
+
+	header_data.channel = channel;
+	header_data.header_name = args.header_name;
+	header_data.header_number = number;
+	header_data.header_value = NULL;
+	header_data.buf = buf;
+	header_data.len = len;
+	header_data.header_datastore = &response_header_datastore;
+
+	if (!strcasecmp(args.action, "read")) {
+		return ast_sip_push_task_wait_serializer(channel->session->serializer, read_header, &header_data);
+	} else {
+		ast_log(AST_LOG_ERROR,
+				"Unknown action '%s' is not valid, must be 'read'.\n",
+				args.action);
+		return -1;
+	}
+}
+
+/*!
+ * \brief Implements PJSIP_HEADER function 'write' callback.
  *
  * Valid actions are 'add', 'update' and 'remove'.
  */
@@ -537,6 +917,7 @@ static int func_write_header(struct ast_channel *chan, const char *cmd, char *da
 	header_data.header_value = value;
 	header_data.buf = NULL;
 	header_data.len = 0;
+	header_data.header_datastore = &header_datastore;
 
 	if (!strcasecmp(args.action, "add")) {
 		return ast_sip_push_task_wait_serializer(channel->session->serializer,
@@ -561,6 +942,21 @@ static struct ast_custom_function pjsip_header_function = {
 	.write = func_write_header,
 };
 
+static struct ast_custom_function pjsip_headers_function = {
+	.name = "PJSIP_HEADERS",
+	.read = func_read_headers
+};
+
+static struct ast_custom_function pjsip_response_header_function = {
+	.name = "PJSIP_RESPONSE_HEADER",
+	.read = func_response_read_header
+};
+
+static struct ast_custom_function pjsip_response_headers_function = {
+	.name = "PJSIP_RESPONSE_HEADERS",
+	.read = func_response_read_headers
+};
+
 /*!
  * \internal
  * \brief Session supplement callback for outgoing INVITE requests
@@ -575,7 +971,6 @@ static struct ast_custom_function pjsip_header_function = {
  */
 static void outgoing_request(struct ast_sip_session *session, pjsip_tx_data * tdata)
 {
-	pj_pool_t *pool = session->inv_session->dlg->pool;
 	struct hdr_list *list;
 	struct hdr_list_entry *le;
 	RAII_VAR(struct ast_datastore *, datastore,
@@ -588,7 +983,7 @@ static void outgoing_request(struct ast_sip_session *session, pjsip_tx_data * td
 
 	list = datastore->data;
 	AST_LIST_TRAVERSE(list, le, nextptr) {
-		pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr *) pjsip_hdr_clone(pool, le->hdr));
+		pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr *) pjsip_hdr_clone(tdata->pool, le->hdr));
 	}
 	ast_sip_session_remove_datastore(session, datastore->uid);
 }
@@ -598,12 +993,16 @@ static struct ast_sip_session_supplement header_funcs_supplement = {
 	.priority = AST_SIP_SUPPLEMENT_PRIORITY_CHANNEL - 1000,
 	.incoming_request = incoming_request,
 	.outgoing_request = outgoing_request,
+	.incoming_response = incoming_response,
 };
 
 static int load_module(void)
 {
 	ast_sip_session_register_supplement(&header_funcs_supplement);
 	ast_custom_function_register(&pjsip_header_function);
+	ast_custom_function_register(&pjsip_headers_function);
+	ast_custom_function_register(&pjsip_response_header_function);
+	ast_custom_function_register(&pjsip_response_headers_function);
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -611,6 +1010,9 @@ static int load_module(void)
 static int unload_module(void)
 {
 	ast_custom_function_unregister(&pjsip_header_function);
+	ast_custom_function_unregister(&pjsip_headers_function);
+	ast_custom_function_unregister(&pjsip_response_header_function);
+	ast_custom_function_unregister(&pjsip_response_headers_function);
 	ast_sip_session_unregister_supplement(&header_funcs_supplement);
 	return 0;
 }
