@@ -24,8 +24,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astdb.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
@@ -145,17 +143,11 @@ enum stasis_mailbox_result stasis_app_mailbox_delete(
 
 static int load_module(void)
 {
-	/* Must be done first */
-	ast_mwi_external_ref();
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
 static int unload_module(void)
 {
-	/* Must be done last */
-	ast_mwi_external_unref();
-
 	return 0;
 }
 
@@ -163,5 +155,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "Stasis applicatio
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis,res_mwi_external"
-	);
+	.requires = "res_stasis,res_mwi_external"
+);
