@@ -18,10 +18,10 @@
 	<support_level>extended</support_level>
  ***/
 
+/* Needed for net-snmp headers */
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 /*
  * There is some collision collision between netsmp and asterisk names,
  * causing build under AST_DEVMODE to fail.
@@ -75,7 +75,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/ast_version.h"
 #include "asterisk/pbx.h"
 
-/* Colission between Net-SNMP and Asterisk */
+/* Collision between Net-SNMP and Asterisk */
 #define unload_module ast_unload_module
 #include "asterisk/module.h"
 #undef unload_module
