@@ -273,8 +273,8 @@ AST_TEST_DEFINE(channel_snapshot_json)
 	ast_test_validate(test, NULL != snapshot);
 
 	actual = ast_channel_snapshot_to_json(snapshot, NULL);
-	expected = ast_json_pack("{ s: s, s: s, s: s, s: s,"
-				 "  s: { s: s, s: s, s: i },"
+	expected = ast_json_pack("{ s: s, s: s, s: s, s: s, s: s,"
+				 "  s: { s: s, s: s, s: i, s: s, s: s },"
 				 "  s: { s: s, s: s },"
 				 "  s: { s: s, s: s },"
 				 "  s: s"
@@ -284,10 +284,13 @@ AST_TEST_DEFINE(channel_snapshot_json)
 				 "state", "Down",
 				 "accountcode", "acctcode",
 				 "id", ast_channel_uniqueid(chan),
+				 "protocol_id", "",
 				 "dialplan",
 				 "context", "context",
 				 "exten", "exten",
 				 "priority", 1,
+				 "app_name", "",
+				 "app_data", "",
 				 "caller",
 				 "name", "cid_name",
 				 "number", "cid_num",
